/*
 * Group.java
 *
 * Created on November 1, 2002, 1:52 PM
 */

package edu.jhu.cer.CollisionGroup;

import java.util.Vector;
import java.util.Stack;

/**
 *
 * @author  gjastrab
 */
public class Group implements Cloneable
{
    private int mySize, id, midPos;
    private double start, end, max;     // these variables are necessary to determine whether points overlap
    private TLPoint earliestPoint, midPoint, latestPoint;
    private Vector myPoints;
    private Vector returnPoints;
    private Stack allPoints;              // to be used when ungrouping points

    public Group(double s, double e, double m)
    {
	mySize = 0;
	id = 0;
	midPos = 0;
	start = s;
	end = e;
	max = m;
	earliestPoint = null;
	midPoint = null;
	latestPoint = null;
	myPoints = new Vector();
	allPoints = new Stack();
	returnPoints = new Vector();
    }
        
    //constructor for clone() method
    public Group(int my, int i, int mi, double s, double e, double m, TLPoint ep, TLPoint mp, TLPoint lp, Vector ps, Stack aps, Vector returnP)
    {
	mySize = my;
	id = i;
	midPos = mi;
	start = s;
	end = e;
	max = m;
	earliestPoint = ep;
	midPoint = mp;
	latestPoint = lp;
	myPoints = ps;
	allPoints = aps;
	returnPoints = returnP;
    }
	
    public boolean isEmpty()
    {
	if(mySize == 0) return true;
	else return false;
    }
	
    public int getSize() { return mySize; }
        
    public void setID(int i) { id = i; }
    public int getID() { return id; }
        
    public Object clone() //overwrites clone method in order to copy a Group object
    {
	return new Group(this.mySize, this.id, this.midPos, this.start, this.end, this.max, this.earliestPoint, this.midPoint, this.latestPoint, this.myPoints, this.allPoints, this.returnPoints);
    }
        
    public void clear()
    {
	myPoints.clear();
	allPoints.clear();
	mySize = 0;
	earliestPoint = null;
	midPoint = null;
	latestPoint = null;
    }
        
    public boolean isOverlapping(double date1, double date2)
    {
	double x1 = ((date1 - start) / (end - start)) * max;
	double x2 = ((date2 - start) / (end - start)) * max;
	if(Math.abs(x1 - x2) < 80) return true;
	else return false;
    }
        
    public void addPoint(TLPoint p)
    {
	if(isEmpty())
            {
                earliestPoint = p;
                midPoint = earliestPoint;
                midPos = 1;
            }
	else if(mySize == 1)  // in case of group of size 2, no true midpoint yet
            {
                latestPoint = p;
                earliestPoint.setNext(latestPoint);
                latestPoint.setPrev(earliestPoint);
            }
	else
            {
                latestPoint.setNext(p);
                p.setPrev(latestPoint);
                latestPoint = p;
            }
            
	// here record within latestPoint and latestPoint.getPrev() if they overlap with each other
	if(mySize > 1 && isOverlapping(latestPoint.getDate(), latestPoint.getPrev().getDate()))
            {
                latestPoint.getPrev().addClash(latestPoint);
                latestPoint.addClash(latestPoint.getPrev());
            }
	myPoints.add(p);
	returnPoints.add(p);
	mySize++;
            
	// now reassign midpoint to appropriate point
	if(midPos <= (mySize / 2))
            {
                midPoint = midPoint.getNext();
                midPos++;
            }
            
	// now check if new midPoint overlaps with earliestPoint
	// (if overlap, loop and remove until the two do not overlap, and group all removed points)
	boolean checker = true;
	while(checker)
            {
                try
		    {
			if(!isOverlapping(midPoint.getDate(), earliestPoint.getDate()))
			    {
				// if the two do not overlap, then remove earliestPoint
				TLPoint temp = earliestPoint;
				earliestPoint = earliestPoint.getNext();
				//temp.setNext(null); take out in order to keep removed points linked in 1-direction
				earliestPoint.setPrev(null);
				removePoint(temp);
				if(midPos <= (mySize / 2)) midPoint = midPoint.getNext();
			    }
			else checker = false;
		    }
                catch(EmptyContainer EC) // temp was not in group (this should never happen)
		    {
			checker = false;
		    }
            }
    }
	
    public void removePoint(TLPoint p) throws EmptyContainer
    {
	int pos = myPoints.indexOf(p);
	if(pos == -1) throw new EmptyContainer();
	TLPoint temp = (TLPoint)myPoints.remove(pos);
	if(allPoints.empty()) allPoints.push(temp);
	else
            {
                if(((TLPoint)allPoints.peek()).isOverlapping(temp)) allPoints.push(temp);
                else
		    {
			allPoints.push("*");
			allPoints.push(temp);
		    }
            }
	midPos--;
	mySize--;
    }
        
    // returns Stack of Points to group (to be done in main) and clears Group
    // **ONLY TO BE CALLED IF POINTS IN MYPOINTS ARE ALL IN THE LAST GROUP**
    public Stack export()
    {
	if(!allPoints.isEmpty()) allPoints.push("*");
	while(!myPoints.isEmpty()) // removes final group and pushes to exporting stack
            {
                allPoints.push(myPoints.remove(0));
            }
	Stack tempStack = new Stack();
	tempStack = (Stack)allPoints.clone();
	clear();
	return tempStack;
    }
    /**
     *arun put this in
     */
        
    public Double[] getPointsInGroup()
    {
	Double[] toReturn = new Double[mySize];
	for(int i = 0;i<returnPoints.size();i++){	
	    toReturn[i]= new Double(((TLPoint)returnPoints.elementAt(i)).getDate());	
	}
        	
	return toReturn;
        
    }
        
        
    public TLPoint first() { return earliestPoint; }
    public TLPoint mid() { return midPoint; }
    public TLPoint last() { return latestPoint; }
}
