package edu.jhu.cer.TimelineCreator;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.ListSelectionModel;

/**
 * Watches for events in the Event Table, like a selection change
 * and updates the correct fields in the General/Detail tabs.
 */

class EventTableSelectionListener implements ListSelectionListener {
    /**
     * The main timeline creator window.
     */
    TimelineCreator tlc;

    /**
     * Saves a reference to the main timeline creator.
     * @see #tlc tlc
     */
    public EventTableSelectionListener(TimelineCreator timeline) {
	tlc = timeline;
    }

    /**
     * Overrides the valueChanged method of ListSelectionListener.
     *
     * @param e ListSelectionEvent passed to function.
     */
    public void valueChanged(ListSelectionEvent e) {
	// if not selecting a row
	if (e.getValueIsAdjusting()) {
	    return;
	}
	ListSelectionModel lsm = (ListSelectionModel)e.getSource();
	
	// if a new row was selected
	if (!lsm.isSelectionEmpty()) {
	    // get which row is being selected
	    int index = lsm.getMinSelectionIndex();

	    // clear all the data in the rows so far
	    clearEventDetails(index);	
	    
	    if (tlc.jt.getValueAt(index,0) != null){tlc.title1TF.setText(tlc.jt.getValueAt(index,0).toString());}
	    if (tlc.jt.getValueAt(index,1) != null){tlc.startTF.setText(tlc.jt.getValueAt(index,1).toString());}
	    if (tlc.jt.getValueAt(index,2) != null){tlc.endTF.setText(tlc.jt.getValueAt(index,2).toString());}
	    if (tlc.jt.getValueAt(index,3) != null){
		    String band = tlc.jt.getValueAt(index,3).toString();
		    int bandindex = 0;
		    if (!band.matches("main")) bandindex = band.substring(3).compareTo("0");

		    tlc.bandCB.setSelectedIndex(bandindex);
	    }
	    if (tlc.jt.getValueAt(index,4) != null){
		Object val = tlc.hm2.get(tlc.jt.getValueAt(index,4).toString()); 
		if(val != null) {
		    tlc.iconCB.setSelectedIndex(((Integer) val).intValue()-1);
		}
	    }
	    if (tlc.jt.getValueAt(index,5) != null && tlc.jt.getValueAt(index,5).toString().toLowerCase().startsWith("y")){tlc.detailCheckB.setSelected(true); } 
	    else{tlc.detailCheckB.setSelected(false);}
	    if (tlc.jt.getValueAt(index,6) != null){tlc.detailText1TF.setText(tlc.jt.getValueAt(index,6).toString());}
	    if (tlc.jt.getValueAt(index,7) != null){tlc.detailText2TF.setText(tlc.jt.getValueAt(index,7).toString());}
	    if (tlc.jt.getValueAt(index,8) != null){tlc.detailText3TF.setText(tlc.jt.getValueAt(index,8).toString());}
	    if (tlc.jt.getValueAt(index,9) != null){tlc.detailText4TF.setText(tlc.jt.getValueAt(index,9).toString());}
	    if (tlc.jt.getValueAt(index,10) != null){tlc.detailImage1TF.setText(tlc.jt.getValueAt(index,10).toString());}
	    if (tlc.jt.getValueAt(index,11) != null){tlc.detailImage2TF.setText(tlc.jt.getValueAt(index,11).toString());}
	    if (tlc.jt.getValueAt(index,12) != null){tlc.detailImage3TF.setText(tlc.jt.getValueAt(index,12).toString());}
	    if (tlc.jt.getValueAt(index,13) != null){tlc.detailImage4TF.setText(tlc.jt.getValueAt(index,13).toString());}
	    if (tlc.jt.getValueAt(index,14) != null){tlc.descriptionTA.setText(tlc.jt.getValueAt(index,14).toString());}
	    if (tlc.jt.getValueAt(index,15) != null && tlc.jt.getValueAt(index,15).toString().toLowerCase().startsWith("y")){tlc.hideCheckB.setSelected(true); }
	    else{tlc.hideCheckB.setSelected(false);}
	    if (tlc.jt.getValueAt(index,16)==null||tlc.jt.getValueAt(index,16).toString().matches(".1.") ){tlc.detailImage1RB.setSelected(true); }
	    else if (tlc.jt.getValueAt(index,16).toString().matches(".2.") ){tlc.detailImage2RB.setSelected(true); }
	    else if (tlc.jt.getValueAt(index,16).toString().matches(".3.") ){tlc.detailImage3RB.setSelected(true); }
	    else if (tlc.jt.getValueAt(index,16).toString().matches(".4.") ){tlc.detailImage4RB.setSelected(true); }
	    else {tlc.detailImage1RB.setSelected(true); }
	
	    int mainimage;
	    mainimage = Integer.parseInt( tlc.jt.getValueAt(index,16).toString() );
	    switch ( mainimage )
	    {
			case 1: tlc.detailImage1RB.setSelected(true); break;
			case 2: tlc.detailImage2RB.setSelected(true); break;
			case 3: tlc.detailImage3RB.setSelected(true); break;
			case 4: tlc.detailImage4RB.setSelected(true); break;
			default: tlc.detailImage1RB.setSelected(true); break;
	    }

	}
    }

    /**
     * Clears all the data in the detail box.
     */
    public void clearEventDetails(int index) {
	tlc.title1TF.setText("");
	tlc.startTF.setText("");
	tlc.endTF.setText("");
	// not sure if this will cause problems
	tlc.bandCB.setSelectedItem("main");
	tlc.iconCB.setSelectedIndex(((Integer) tlc.hm2.get("bomb")).intValue()-1);
	tlc.detailCheckB.setSelected(false); 
	tlc.detailText1TF.setText("");
	tlc.detailText2TF.setText("");
	tlc.detailText3TF.setText("");
	tlc.detailText4TF.setText("");
	tlc.detailImage1TF.setText("");
	tlc.detailImage2TF.setText("");
	tlc.detailImage3TF.setText("");
	tlc.detailImage4TF.setText("");
	tlc.descriptionTA.setText("");
	tlc.hideCheckB.setSelected(false);
    }
}
