package edu.jhu.cer.TimelineCreator;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFileChooser;
import javax.swing.JTable;

/**
 * Opens a window to choose a timeline XML file to edit with the timeline creator tool.
 * After selecting the file, a DocumentBuilder object attempts to parse it as valid
 * timeline xml. If the file is valid timeline xml, that data is passed to a {@link TimelineDataReader}
 * object that puts all the data in the right place. If not, a warning window is popped up indicating
 * to the user that they have selected a file of an incorrect type and that they should try again.
 */
class OpenMenuHandler implements ActionListener {
    /**
     * The main timeline creator window.
     */
    TimelineCreator tlc;

    /**
     * The table used to store all the event data back in the {@link TimelineCreator}.
     */
    JTable jt;

    /**
     * Saves a reference to the main timeline creator.
     * @see #tlc tlc
     */
    public OpenMenuHandler(TimelineCreator timeline, JTable table) {
	tlc = timeline;
	jt = table;
    }

    /**
     * Shows a file chooser window for the user to select which timeline xml file
     * they would like to open for editing.
     */
    public void actionPerformed(ActionEvent ae) {
	JFileChooser jfc = new JFileChooser(System.getProperty("user.dir"));

	// open the file chooser window with the main Timeline window as a parent to provide
	// consistent look and feel.
	int returnVal = jfc.showOpenDialog(tlc);
  		
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    
	    String filename = jfc.getSelectedFile().getAbsolutePath();
	    tlc.currentFilePath = filename;
	    tlc.currentFileName = jfc.getSelectedFile().getName();

	    // pass this to the TimelineDataReader
	    TimelineDataReader tdr = new TimelineDataReader(tlc, jt);
	    tdr.readData(filename);
	    tlc.setBands();
	}
    }
}
