package edu.jhu.cer.TimelineCreator;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Arrays;
import javax.swing.JFileChooser;
import javax.swing.JTable;
import java.io.File;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.io.IOException;

/**
 * Handler to save data to HTML.
 */
class SaveHTMLMenuHandler implements ActionListener {
    
    HashMap mainHM = new HashMap();
    HashMap sub1HM = new HashMap();
    HashMap sub2HM = new HashMap();
    HashMap sub3HM = new HashMap();
    HashMap sub4HM = new HashMap();
    HashMap sub5HM = new HashMap();

    /**
     * The main timeline creator window.
     */
    TimelineCreator tlc;

    /**
     * The table used to store all the event data back in the {@link TimelineCreator}.
     */
    JTable jt;

    /**
     * Saves a reference to the main timeline creator.
     * @see #tlc tlc
     */
    public SaveHTMLMenuHandler(TimelineCreator timeline, JTable table) {
	tlc = timeline;
	jt = table;
    }
    
    /**
     * Public method for class.
     */
    public void actionPerformed(ActionEvent ae) {
	JFileChooser jfc = new JFileChooser(System.getProperty("user.dir"));
	File myfile = new File("timeline_preview.html");
	jfc.setSelectedFile(myfile);
	int choice = jfc.showSaveDialog(tlc);
	if (choice == JFileChooser.APPROVE_OPTION) {
	    String filename = jfc.getSelectedFile().getAbsolutePath();
      	   	if(jfc.getSelectedFile().exists()){
		    
		    WarnOverwriteHTML woh = new WarnOverwriteHTML(jfc.getSelectedFile());
		    
		}
		
		if(tlc.proceedOverwriteHTML){
		    
		    
		    int rows = jt.getRowCount() - 1;
		    for(int i=0;i<=rows;i++){
			if(jt.getValueAt(i,3).toString().equals("main")){
			    mainHM.put(new Integer(i),new Double(jt.getValueAt(i,1).toString()));
			}
			else if(jt.getValueAt(i,3).toString().equals("sub1")){
			    sub1HM.put(new Integer(i),new Double(jt.getValueAt(i,1).toString()));
			}
			else if(jt.getValueAt(i,3).toString().equals("sub2")){
			    sub2HM.put(new Integer(i),new Double(jt.getValueAt(i,1).toString()));
			}
			else if(jt.getValueAt(i,3).toString().equals("sub3")){
			    sub3HM.put(new Integer(i),new Double(jt.getValueAt(i,1).toString()));
			}
			else if(jt.getValueAt(i,3).toString().equals("sub4")){
			    sub4HM.put(new Integer(i),new Double(jt.getValueAt(i,1).toString()));
			}
			else if(jt.getValueAt(i,3).toString().equals("sub5")){
			    sub5HM.put(new Integer(i),new Double(jt.getValueAt(i,1).toString()));
			}
		    }

		    try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(filename));
			//String newline = System.getProperty("line.separator");
			//bw buffered writer, writes as data is read
			bw.write("<html>\n");
			bw.write("<head>\n");
			bw.write("<!-- This page was automatically generated by the Timeline Creator tool. http://timeline.cer.jhu.edu -->\n");
			bw.write("<title>" + tlc.configTitleTF.getText() + "</title>\n");
			bw.write("</head>\n");
			bw.write("<body>\n");
			bw.write("<h1>" + tlc.configTitleTF.getText() + "</h1>\n");
           	
			if(sub1HM.size()>=1)
			    bw.write("<a href=\"#sub1\">"+tlc.configBand1TitleTF.getText()+"</a>&nbsp;");
				
			if(sub2HM.size()>=1)
			    bw.write("<a href=\"#sub2\">"+tlc.configBand2TitleTF.getText()+"</a>&nbsp;");
				
			if(sub3HM.size()>=1)
			    bw.write("<a href=\"#sub3\">"+tlc.configBand3TitleTF.getText()+"</a>&nbsp;");
				
			if(sub4HM.size()>=1)
			    bw.write("<a href=\"#sub4\">"+tlc.configBand4TitleTF.getText()+"</a>&nbsp;");
				
			if(sub5HM.size()>=1)
			    bw.write("<a href=\"#sub5\">"+tlc.configBand5TitleTF.getText()+"</a>&nbsp;");
				
			bw.write("<hr/>");			
			
			printDiffBands(mainHM,bw);
			
			if(sub1HM.size()>=1){
			    bw.write("<hr/>");			
			    bw.write("<a name=\"sub1\"><h2>"+tlc.configBand1TitleTF.getText()+"</h2></a>");
			    printDiffBands(sub1HM,bw);
			}
			if(sub2HM.size()>=1){
			    bw.write("<hr/>");			
			    bw.write("<a name=\"sub2\"><h2>"+tlc.configBand2TitleTF.getText()+"</h2></a>");
			    printDiffBands(sub2HM,bw);
			}
			if(sub3HM.size()>=1){
			    bw.write("<hr/>");			
			    bw.write("<a name=\"sub3\"><h2>"+tlc.configBand3TitleTF.getText()+"</h2></a>");
			    printDiffBands(sub3HM,bw);
			}
			if(sub4HM.size()>=1){
			    bw.write("<hr/>");			
			    bw.write("<a name=\"sub4\"><h2>"+tlc.configBand4TitleTF.getText()+"</h2></a>");
			    printDiffBands(sub4HM,bw);
			}
			if(sub5HM.size()>=1){
			    bw.write("<hr/>");			
			    bw.write("<a name=\"sub5\"><h2>"+tlc.configBand5TitleTF.getText()+"</h2></a>");
			    printDiffBands(sub5HM,bw);
			}	

			bw.write("<hr><small>"+tlc.configCopyrightTF.getText()+"</small>"); 
			bw.write("</body>\n");
			bw.write("</html>");
			bw.flush();
			bw.close();
		    } catch (IOException ioe) {};
		    //this last section closes file
		} 
	    }
	}

    /**
     * Function to print data across different bands.
     *
     * @param toPrint HashMap to print from
     *        bw BufferedWriter used to write
     */
    public void printDiffBands(HashMap toPrint,BufferedWriter bw){
	
	Object[] datesArray = toPrint.values().toArray();
	Arrays.sort(datesArray);
	
	String detailTitle1 = tlc.configDetail1TitleTF.getText();
	String detailTitle2 = tlc.configDetail2TitleTF.getText();
	String detailTitle3 = tlc.configDetail3TitleTF.getText();
	String detailTitle4 = tlc.configDetail4TitleTF.getText();
	
	int rows = jt.getRowCount() - 1;
	for(int i = 0;i <= rows; i++) {
	    
	    try{
		
		for(int k = 0;k<datesArray.length;k++){
		    if(toPrint.containsKey(new Integer(i))){
			
			if(new Double(toPrint.get(new Integer(i)).toString()).equals(new Double(datesArray[k].toString()))){
			    toPrint.remove(new Integer(i));
			    
			    if(jt.getValueAt(i,15).toString().equalsIgnoreCase("no")){
                                for(int j = 0; j <= 11; j++)
                                    if(jt.getValueAt(i, j) == null) jt.setValueAt("", i, j);
				bw.write("<h3>"+jt.getValueAt(i,0)+"</h3>\n");	
				if( ((String)jt.getValueAt(i,10)).length() > 0 )
				{
					bw.write("<B>Image: </B>\n");
					bw.write("<a href=\"file:///"+jt.getValueAt(i,10)+"\" target=\"_new\">"+jt.getValueAt(i,10)+"</a>\n<br>\n");
				}
				if( ((String)jt.getValueAt(i,11)).length() > 0 )
				{
					bw.write("<B>Image: </B>\n");
					bw.write("<a href=\"file:///"+jt.getValueAt(i,11)+"\" target=\"_new\">"+jt.getValueAt(i,11)+"</a>\n<br>\n");
				}
				if( ((String)jt.getValueAt(i,12)).length() > 0 )
				{
					bw.write("<B>Image: </B>\n");
					bw.write("<a href=\"file:///"+jt.getValueAt(i,12)+"\" target=\"_new\">"+jt.getValueAt(i,12)+"</a>\n<br>\n");
				}
				if( ((String)jt.getValueAt(i,13)).length() > 0 )
				{
					bw.write("<B>Image: </B>\n");
					bw.write("<a href=\"file:///"+jt.getValueAt(i,13)+"\" target=\"_new\">"+jt.getValueAt(i,13)+"</a>\n<br>\n");
				}

				bw.write("<b>Event date(s)</b> : " + jt.getValueAt(i,1));
                                if(jt.getValueAt(i,2) == "")
                                    bw.write("<br>\n");
                                else
                                    bw.write(" - " + jt.getValueAt(i,2) + "<br>\n");
				if(jt.getValueAt(i,5).toString().equalsIgnoreCase("no")){
				    bw.write("<b>"+detailTitle1+"</b> : "+jt.getValueAt(i,6)+"<br>\n");
				    bw.write("<b>"+detailTitle2+"</b> : "+jt.getValueAt(i,7)+"<br>\n");
				    bw.write("<b>"+detailTitle3+"</b> : "+jt.getValueAt(i,8)+"<br>\n");
				    bw.write("<b>"+detailTitle4+"</b> : "+jt.getValueAt(i,9)+"<br>\n");
				    bw.write("<b>Description</b> : "+jt.getValueAt(i,11)+"<br>\n");
				}
				bw.write("<br><br>\n");
			    }
			}
		    }
		}
	    } catch (IOException ioe) {};
	}
    }
}
