package edu.jhu.cer.TimelineCreator;

import java.awt.*;            //abstract window toolkit
import java.awt.event.*;      //awt event handler 
import javax.swing.*;         //enlarged awt 1.3
import javax.swing.table.*;   //table package for top portion of general tab
import java.io.*;             //input output
import java.util.*;           //string tokenizer to read tab separated import file
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.filechooser.*;
import javax.swing.border.*;

import javax.xml.parsers.*;   //xml pkg 1.4
import org.xml.sax.*;         //xml pkg 1.4
import org.xml.sax.helpers.*; //xml pkg 1.4
import org.w3c.dom.*;         //xml pkg 1.4

//import edu.jhu.cer.TimelineCreator.*;

// JFrame is the container which is filled with all of the application specfic menus, 
// and other UI items, Itemlister is what provides interactivity

/**
 * The main class in the TimelineCreator package, this holds all the UI information,
 * buttons, panels, handlers and the like.
 */

public class TimelineCreator extends JFrame{
    ///////////////////////////
    // variable declarations //
    ///////////////////////////

    /******************/
    /** the menu bar **/
    /******************/

    /**
     * Holds the various {@link JMenu} items making up the File, Preview and Save menus.
     */
    JMenuBar jmb;

    /**
     * Basic JMenu object to hold menu items.
     */
    //previewMenu commented out
    JMenu fileMenu,viewMenu,helpMenu;

    /**
     * Element of the File menu.
     */
    //exportMenuItem commented out from JMenuItem list.
    JMenuItem quitMenuItem, importMenuItem, openMenuItem, saveMenuItem, saveHTMLMenuItem, navigationMenuItem;
    /**
     * Element of the Preview menu.
     */
    //JMenuItem previewMenuItem;
    /**
     * Element of the Help menu.
     */
    JMenuItem helpTopicsMenuItem, aboutMenuItem;

    /**********************************/
    /** The table to hold event data **/
    /**********************************/

    /**
     * The table object which holds event data in rows of 13 columns.
     */
    JTable jt;

    /**
     * The actual table model. This determines how many rows and columns the table
     * starts out with in addition to containing an {@link ActionListener} for the table.
     */
    DefaultTableModel dtm;

    /**
     * Used to indicate that the default table column model will be used instead of defining one of our own.
     */
    DefaultTableColumnModel dtcm;

    /**
     * Column in the table {@link #jt jt}.
     */
    TableColumn column0,column1,column2,column3,column4,column5,column6,column7,column8,column9,column10,column11,column12,column13,column14,column15,column16;

    /********************************/
    /** The Event item detail pane **/
    /********************************/

    /**
     * Label in the Event item detail pane.
     */
    JLabel generalTabLabel, detailTabLabel, title1Label, iconLabel, startLabel, endLabel, bandLabel, detailImage1Label, detailImage2Label, detailImage3Label, detailImage4Label, detailText1Label, detailText2Label, detailText3Label, detailImageBGLabel, detailText4Label, descriptionLabel;

    /**
     * Text field in the event item detail pane.
     */
    JTextField title1TF, startTF, endTF, detailImage1TF, detailImage2TF, detailImage3TF, detailImage4TF, detailText1TF, detailText2TF, detailText3TF, detailText4TF;

    /**
     * The description box in the event item detail pane.
     */
    JTextArea descriptionTA;

    /**
     * Combo box for selecting which band an event belongs to.
     */
    JComboBox bandCB;

    /**
     * Check box for the event.
     */
    JCheckBox hideCheckB, detailCheckB;

    /**
     * Pane in the first panel.
     */
    JTabbedPane mainTabbedPane, dataTabbedPane;
    /**
     * Radio buttons to select main image
     */
    JRadioButton detailImage1RB, detailImage2RB, detailImage3RB, detailImage4RB;
    /**
     * Radio button group for selecting image
     */
    ButtonGroup detailImageBG;

    /*****************/
    /** Panel Stuff **/
    /*****************/

    /**
     * Panel for the TimelineCreator.
     */
    JPanel generalDataPanel, detailDataPanel, buttonPanel, containerPanel, dataEditPanel;

    /**
     * Button used for event data modification
     */
    JButton addButton,deleteButton,nextButton,prevButton,firstButton,lastButton,replaceButton,generalTabHelpButton, detailTabHelpButton, detailImage1Button, detailImage2Button, detailImage3Button, detailImage4Button;
    
    /**
     * Column headers.
     */
    JTableHeader jth;

    /**
     * Row selection model and listener
     */
    ListSelectionModel rowSM;

    /**
     * Variable to hold GridBagConstraints created with each button and such.
     */
    GridBagConstraints gbc;

    /**
     * Scroll pane.
     */
    JScrollPane tableScroll, descriptionSP;

    /******************/
    /** Config Panel **/
    /******************/

    /**
     * The main panel for general timeline configuration options.
     */
    JPanel configPanel; 

    /**
     * Label used in the config panel.
     */
    JLabel configTabLabel, configTitleLabel,configStartLabel,configStopLabel,configUnitsLabel, configSpacingLabel,configBand1TitleLabel,configBand1ColorLabel,configBand2TitleLabel,configBand2ColorLabel,configBand3TitleLabel,configBand3ColorLabel,configBand4TitleLabel,configBand4ColorLabel,configBand5TitleLabel,configBand5ColorLabel,configDetail1TitleLabel,configDetail2TitleLabel,configDetail3TitleLabel,configDetail4TitleLabel,configBGColorLabel,configBorderColorLabel,configTextColorLabel,configTextFontLabel,configCopyrightLabel;

    /**
     * Text field in the config panel
     */
    JTextField configTitleTF,configStartTF,configStopTF,configUnitsTF, configSpacingTF,configBand1TitleTF,configBand2TitleTF,configBand3TitleTF,configBand4TitleTF,configBand5TitleTF,configDetail1TitleTF,configDetail2TitleTF,configDetail3TitleTF,configDetail4TitleTF,configCopyrightTF;

    /**
     * Button in the config panel
     */
    JButton configTabHelpButton, configBand2ColorButton,configBand1ColorButton,configBand3ColorButton,configBand4ColorButton,configBand5ColorButton,configBGColorButton,configBorderColorButton,configTextColorButton;

    /**
     * Combo Box used in the config panel.
     */
    JComboBox configTextFontCB,iconCB;

    /**
     * Hashmap objects to store image icon names.
     */
    HashMap hm2;

    /**
     * An array to hold image icons.
     */
    ImageIcon stuff[];


    public static boolean proceed = true;
    public static boolean proceedOverwrite = true;
    public static boolean proceedOverwriteHTML = true;
  
    String title = "Timeline Creator";

    /**
     * Used to store just the file name of the current file either being read or written.
     */ 
    String currentFileName = "";

    /**
     * Used to store both the file name and the path before it of the current file.
     */
    String currentFilePath = "";

    /**
     * This constructor sets up all of the UI elements used by the TimelineCreator.
     */
    public TimelineCreator() {
     
	hm2 = new HashMap(12);
  
	hm2.put("bomb",new Integer(1));
	hm2.put("book",new Integer(2));
	hm2.put("bulb",new Integer(3));
	hm2.put("circle",new Integer(4));
	hm2.put("clock",new Integer(5));
	hm2.put("diamond",new Integer(6));
	hm2.put("hourglass",new Integer(7));
	hm2.put("key",new Integer(8));
	hm2.put("music",new Integer(9));
	hm2.put("square",new Integer(10));
	hm2.put("star",new Integer(11));
	hm2.put("range",new Integer(12));
     
	//define colums for general tab, also start with 0, in this case 14 columns 0-13
	column0 = new TableColumn(0);
	column0.setHeaderValue("Title");
	column0.setPreferredWidth(185);
    
	column1 = new TableColumn(1);
	column1.setHeaderValue("Start");
	column1.setPreferredWidth(55);
    
	column2 = new TableColumn(2);
	column2.setHeaderValue("End");
	column2.setPreferredWidth(55);
    
	column3 = new TableColumn(3);
	column3.setHeaderValue("Band");
	column3.setPreferredWidth(87);  

	column4 = new TableColumn(4);
	column4.setHeaderValue("Icon");
	column4.setPreferredWidth(75);
    
	column5 = new TableColumn(5);
	column5.setHeaderValue("Hide Detail");
	column5.setPreferredWidth(75);
    
	column6 = new TableColumn(6);
	column6.setHeaderValue("Detail Text 1");
	column6.setPreferredWidth(150);
    
	column7 = new TableColumn(7);
	column7.setHeaderValue("Detail Text 2");
	column7.setPreferredWidth(150);
    
	column8 = new TableColumn(8);
	column8.setHeaderValue("Detail Text 3");
	column8.setPreferredWidth(150);
    	
	column9 = new TableColumn(9);
	column9.setHeaderValue("Detail Text 4");
	column9.setPreferredWidth(150);
    
	column10 = new TableColumn(10);
	column10.setHeaderValue("Media Item 1");
	column10.setPreferredWidth(100);
    
	column11 = new TableColumn(11);
	column11.setHeaderValue("Media Item 2");
	column11.setPreferredWidth(100);
	
	column12 = new TableColumn(12);
	column12.setHeaderValue("Media Item 3");
	column12.setPreferredWidth(100);
	
	column13 = new TableColumn(13);
	column13.setHeaderValue("Media Item 4");
	column13.setPreferredWidth(100);
	
	column14 = new TableColumn(14);
	column14.setHeaderValue("Description");
	column14.setPreferredWidth(200);
    
	column15 = new TableColumn(15);
	column15.setHeaderValue("Hide Event");
	column15.setPreferredWidth(75);
	
	column16 = new TableColumn(16);
	column16.setHeaderValue("Main Image");
	column16.setPreferredWidth(75);
    
	//declaring that the default Column Model will be used, rather than creating our own
	dtcm = new DefaultTableColumnModel();
	dtm = new DefaultTableModel(0, 17);  //start off with 0 rows and 13 columns
    
	dtm.addTableModelListener(new IsTableEdited());
    
	jt = new JTable(dtm,dtcm);  //jTable is what is used to reference table locations
	jt.addColumn(column0);
	jt.addColumn(column1);
	jt.addColumn(column2);
	jt.addColumn(column3);
	jt.addColumn(column4);
	jt.addColumn(column5);
	jt.addColumn(column6);
	jt.addColumn(column7);
	jt.addColumn(column8);
	jt.addColumn(column9);
	jt.addColumn(column10);
	jt.addColumn(column11);
	jt.addColumn(column12);
	jt.addColumn(column13);
	jt.addColumn(column14);
	jt.addColumn(column15);
	jt.addColumn(column16);
    
	jth = new JTableHeader(dtcm);  //adds colums headers
	jt.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);  //only one row can be selected at a time
	jt.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);  //autoresize off forces the horizontal scroll to be present, so that we could choose which columns are shown by default
	jt.setGridColor(Color.gray);
	jt.getTableHeader().setReorderingAllowed(false);
	//jt.setGridColor(Color.black);
    
	rowSM = jt.getSelectionModel();  //when a row is highlighted diplay the selected values in the form
	rowSM.addListSelectionListener(new EventTableSelectionListener(this));    
	//menu items must be created before can be added to menu    
	quitMenuItem = new JMenuItem("Quit");
	quitMenuItem.setAccelerator(KeyStroke.getKeyStroke('Q', Event.CTRL_MASK, false));
	quitMenuItem.addActionListener(new QuitMenuHandler(this));  //tell what to do when menu item is clicked

	// pass a reference for this item to the open menu handler
	// so it can reference this base item when adding data from an xml file

	openMenuItem = new JMenuItem("Open");
	openMenuItem.setAccelerator(KeyStroke.getKeyStroke('O', Event.CTRL_MASK, false));
	openMenuItem.addActionListener(new OpenMenuHandler(this, jt));
    
	importMenuItem = new JMenuItem("Import");
	importMenuItem.setAccelerator(KeyStroke.getKeyStroke('I', Event.CTRL_MASK, false));
	importMenuItem.addActionListener(new ImportMenuHandler(this, jt, dtm));
    
	/*exportMenuItem = new JMenuItem("Export");
	exportMenuItem.setAccelerator(KeyStroke.getKeyStroke('E', Event.CTRL_MASK, false));
	exportMenuItem.addActionListener(new ExportMenuHandler(this, jt));*/
    	
	// see the note above the open menuhandler

	saveMenuItem = new JMenuItem("Save");
	saveMenuItem.setAccelerator(KeyStroke.getKeyStroke('S', Event.CTRL_MASK, false));
	saveMenuItem.addActionListener(new SaveMenuHandler(this));

	saveHTMLMenuItem = new JMenuItem("Save as HTML");
	saveHTMLMenuItem.setAccelerator(KeyStroke.getKeyStroke('H', Event.CTRL_MASK, false));
	saveHTMLMenuItem.addActionListener(new SaveHTMLMenuHandler(this, jt));

	//creates menu and adds previously declared menu items to the menu, also sets order for menu items
	fileMenu = new JMenu("File");
	fileMenu.add(openMenuItem);
	fileMenu.add(saveMenuItem);
	
	//creates a view menu to control navigation butotns
	navigationMenuItem = new JMenuItem("Hide/Show Navigation Buttons");
	navigationMenuItem.setAccelerator(KeyStroke.getKeyStroke('N', Event.CTRL_MASK, false));
	navigationMenuItem.addActionListener(new ViewMenuHandler(this));
	
	viewMenu = new JMenu("View");
	viewMenu.add(navigationMenuItem);
	
	
	
	// CSV NOTES //
	// We have decided to drop support for importing or exporting csv files
	// because we don't really find them useful. People may want to have comments
	// on events that include commas, and that messes up writing or reading csv files
	// plus, we don't even have any idea how many people are actually use this feature.

	//fileMenu.add(importMenuItem);
	//fileMenu.add(exportMenuItem);
	fileMenu.add(saveHTMLMenuItem);
	fileMenu.add(new JSeparator()); //separator between menu items
	fileMenu.add(quitMenuItem);

      
	/*previewMenu = new JMenu("Preview");
    
	previewMenuItem = new JMenuItem("Preview in browser");
	previewMenuItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_ENTER,Event.CTRL_MASK, false));
	previewMenuItem.addActionListener(new PreviewMenuHandler(this));	
    
	previewMenu.add(previewMenuItem);*/
    
	helpMenu = new JMenu("Help");
    
	helpTopicsMenuItem = new JMenuItem("Help Topics");
	helpTopicsMenuItem.addActionListener(new HelpTopicsMenuHandler());
    
	aboutMenuItem = new JMenuItem("About Timeline Creator");
	aboutMenuItem.addActionListener(new AboutMenuHandler(this));       
    
	helpMenu.add(helpTopicsMenuItem);
	helpMenu.add(aboutMenuItem);

	jmb = new JMenuBar();//parent menu bar for file, publish, and help menus
	jmb.add(fileMenu);
	jmb.add(viewMenu);
	//jmb.add(previewMenu);
	jmb.add(helpMenu);

    	generalTabLabel = new JLabel("<html><b>These data will be located in the main band</b></html>");  //used html for formatting - bold
	title1Label = new JLabel("Title: ");

	startLabel = new JLabel("Start: ");
	endLabel = new JLabel("End: ");
	bandLabel = new JLabel("Band: ");
    
	String[] bandnum = {"main","sub1","sub2","sub3","sub4","sub5"};  //options for band drop box
	//CHANGE THIS INFO IN TWO OTHER PLACES BELOW
    
    
	bandCB = new JComboBox(bandnum);  //drop down list
    
	//declares text field size
	title1TF = new JTextField(20);
	startTF = new JTextField(6);
	endTF = new JTextField(6);

	hideCheckB = new JCheckBox("Hide Event");
	detailCheckB = new JCheckBox("Hide Detail");
	//detailCheckB.addItemListener(this);
    
	generalTabHelpButton = new JButton("<html><b>Help?</b></html>");
	generalTabHelpButton.setForeground(Color.red);
	generalTabHelpButton.addActionListener(new GenericHelp());
    
	iconLabel = new JLabel("Icon: ");

	//Vector stuff = new Vector();

	//    File parentDir = new File("thumbnails");
	//    String[] files = parentDir.list();
	//    for(int myI = 0;myI<files.length;myI++){
	//    	stuff.addElement(new ImageIcon("thumbnails/"+files[myI]));
	//    }	

	//Set mySet = hm.keySet();
	//Object[] files = mySet.toArray();

	//    for(int myI = 1;myI<=11;myI++){
	//    	stuff.addElement(new ImageIcon(hm.get(new Integer(myI)).toString()));
	//    	//System.out.println(hm.get(new Integer(myI)).toString());
	//    }
    
	String[] imageFiles = {"bomb", "book", "bulb", "circle", "clock","diamond","hourglass","key","music","square","star","range"};
	stuff = new ImageIcon[imageFiles.length];
 
	for (int myI = 0; myI < imageFiles.length; myI++) {
	    ClassLoader cl1 = getClass().getClassLoader();
	    stuff[myI] = new ImageIcon(cl1.getResource("thumbnails/" + imageFiles[myI] + ".jpg"));
	    stuff[myI].setDescription(imageFiles[myI]);
	}
        
	iconCB = new JComboBox(stuff);
	ComboBoxRenderer renderer= new ComboBoxRenderer();
	iconCB.setRenderer(renderer);
    
	iconCB.setSelectedIndex(9);
    
	generalDataPanel = new JPanel();//creates General Tab/Panel then you add components
	detailDataPanel = new JPanel();//creates Detail Tab/Panel then you add components
	//create components, create panels, add components to panels, add panels to tabs

	//formats the forms for user input/editing
	GridBagLayout gbl = new GridBagLayout();
	generalDataPanel.setLayout(gbl);
	//(which column, which row, how long, how tall, always zero, always zero
	//add labels, then text boxes, all by row
	//chose to use gbc as the only variable because it is only read once, improves performance
    
	//Insets i = new Insets(2,7,3,7);  //these are default insets, insets are synonomous with cell padding
Insets i = new Insets(1,2,1,2);
	gbc = new GridBagConstraints(0, 0, 5, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0,10,10,0), 0, 0);
	gbl.setConstraints(generalTabLabel, gbc);

	gbc = new GridBagConstraints(5, 0, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0,10,10,10), 0, 0);
	gbl.setConstraints(generalTabHelpButton, gbc);

	gbc = new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, i, 0, 0);
	gbl.setConstraints(title1Label, gbc);

	gbc = new GridBagConstraints(1, 1, 5, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,7,3,10), 0, 0);
	gbl.setConstraints(title1TF, gbc);
     
	gbc = new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, i, 0, 0);
	gbl.setConstraints(bandLabel, gbc);

	gbc = new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, i, 0, 0);
	gbl.setConstraints(bandCB, gbc);

	gbc = new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, i, 0, 0);
	gbl.setConstraints(startLabel, gbc);

	gbc = new GridBagConstraints(3, 2, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, i, 0, 0);
	gbl.setConstraints(startTF, gbc);

	gbc = new GridBagConstraints(4, 2, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(2,9,3,7), 0,0);
	gbl.setConstraints(endLabel, gbc);

	gbc = new GridBagConstraints(5, 2, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,new Insets(2,9,3,7), 0, 0);
	gbl.setConstraints(endTF, gbc);        

	gbc = new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,9,90,7), 0,0);
	gbl.setConstraints(iconLabel, gbc);

	gbc = new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,7,90,7), 0, 0);
	gbl.setConstraints(iconCB, gbc);        
     		
	gbc = new GridBagConstraints(2, 3, 2, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2,30,90,7), 0, 0);
	gbl.setConstraints(detailCheckB, gbc);    

	gbc = new GridBagConstraints(4, 3, 2, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2,30,90,7), 0, 0);
	gbl.setConstraints(hideCheckB, gbc);    
    
	//all components are now being added to panel
	generalDataPanel.add(generalTabLabel);
	generalDataPanel.add(generalTabHelpButton);
	generalDataPanel.add(title1Label);
	generalDataPanel.add(title1TF);
	generalDataPanel.add(startLabel);
	generalDataPanel.add(startTF);
	generalDataPanel.add(endLabel);
	generalDataPanel.add(endTF);
	generalDataPanel.add(bandLabel);
	generalDataPanel.add(bandCB);  
	generalDataPanel.add(iconLabel);
	generalDataPanel.add(iconCB);
	generalDataPanel.add(detailCheckB);
	generalDataPanel.add(hideCheckB);
	
	//repeat of above process for detail panel
	detailTabLabel = new JLabel("<html><b>These data will be located in the detail window</b></html>");   
	detailImage1Label = new JLabel("Media Item 1: ");
	detailImage2Label = new JLabel("Media Item 2: ");
	detailImage3Label = new JLabel("Media Item 3: ");
	detailImage4Label = new JLabel("Media Item 4: ");
	detailText1Label = new JLabel("Detail Text 1: ");
	detailText2Label = new JLabel("Detail Text 2: ");
	detailText3Label = new JLabel("Detail Text 3: ");
	detailText4Label = new JLabel("Detail Text 4: ");
	descriptionLabel = new JLabel("Description: "); 
    
	detailImage1Button = new JButton("Browse");
	detailImage1Button.addActionListener(new DetailBrowse(this,1));
	detailImage2Button = new JButton("Browse");
	detailImage2Button.addActionListener(new DetailBrowse(this,2));
	detailImage3Button = new JButton("Browse");
	detailImage3Button.addActionListener(new DetailBrowse(this,3));
	detailImage4Button = new JButton("Browse");
	detailImage4Button.addActionListener(new DetailBrowse(this,4));
    
	detailImage1TF = new JTextField(29);
	detailImage2TF = new JTextField(29);
	detailImage3TF = new JTextField(29);
	detailImage4TF = new JTextField(29);
	detailText1TF = new JTextField(30);
	detailText2TF = new JTextField(30);
	detailText3TF = new JTextField(30);
	detailText4TF = new JTextField(30);
   
	descriptionTA = new JTextArea(8,30);
	descriptionTA.setLineWrap(true);
	//descriptionTA.setWrapStyleWord(true);
	
	// fonts?
	//descriptionTA.setFont(new Font("Batang", Font.PLAIN, 12));

	detailTabHelpButton = new JButton("<html><b>Help?</b></html>");
	detailTabHelpButton.setForeground(Color.red);
	detailTabHelpButton.addActionListener(new GenericHelp());
    
	descriptionSP = new JScrollPane(descriptionTA,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
	
	detailDataPanel.setLayout(gbl);
    
	gbc = new GridBagConstraints(0, 0, 13, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5,10,10,0), 0, 0);
	gbl.setConstraints(detailTabLabel, gbc);
    
	gbc = new GridBagConstraints(13, 0, 1, 1, 0.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(5,10,10,5), 0, 0);
	gbl.setConstraints(detailTabHelpButton, gbc);
    
	gbc = new GridBagConstraints(0, 1, 2, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailText1Label, gbc);
	
	gbc = new GridBagConstraints(2, 1, 12, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,5), 0, 0);
	gbl.setConstraints(detailText1TF, gbc);

	gbc = new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailText2Label, gbc);
	
	gbc = new GridBagConstraints(2, 2, 12, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,5), 0, 0);
	gbl.setConstraints(detailText2TF, gbc);
    
	gbc = new GridBagConstraints(0, 3, 2, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailText3Label, gbc);

	gbc = new GridBagConstraints(2, 3, 12, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,5), 0, 0);
	gbl.setConstraints(detailText3TF, gbc);    

	gbc = new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE,new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailText4Label, gbc);

	gbc = new GridBagConstraints(2, 4, 12, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,5), 0, 0);
	gbl.setConstraints(detailText4TF, gbc);    


	//image 1
	gbc = new GridBagConstraints(0, 5, 2, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailImage1Label, gbc);

	gbc = new GridBagConstraints(2, 5, 11, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,3), 0, 0);
	gbl.setConstraints(detailImage1TF, gbc);    

	gbc = new GridBagConstraints(13, 5, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,5), 0, 0);
	gbl.setConstraints(detailImage1Button, gbc);    

	//image 2
	gbc = new GridBagConstraints(0, 6, 2, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailImage2Label, gbc);

	gbc = new GridBagConstraints(2, 6, 11, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,3), 0, 0);
	gbl.setConstraints(detailImage2TF, gbc);    

	gbc = new GridBagConstraints(13, 6, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,5), 0, 0);
	gbl.setConstraints(detailImage2Button, gbc);  
	
	//image 3
	gbc = new GridBagConstraints(0, 7, 2, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailImage3Label, gbc);

	gbc = new GridBagConstraints(2, 7, 11, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,3), 0, 0);
	gbl.setConstraints(detailImage3TF, gbc);    

	gbc = new GridBagConstraints(13, 7, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,5), 0, 0);
	gbl.setConstraints(detailImage3Button, gbc);  
	
	//image 4
	gbc = new GridBagConstraints(0, 8, 2, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailImage4Label, gbc);

	gbc = new GridBagConstraints(2, 8, 11, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,3), 0, 0);
	gbl.setConstraints(detailImage4TF, gbc);    

	gbc = new GridBagConstraints(13, 8, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2,0,2,5), 0, 0);
	gbl.setConstraints(detailImage4Button, gbc);  
	
	//pick images
	detailImage1RB = new JRadioButton("Item 1");
	detailImage1RB.setActionCommand("1");
	detailImage1RB.setSelected(true);
	detailImage2RB = new JRadioButton("Item 2");
	detailImage2RB.setActionCommand("2");
	detailImage3RB = new JRadioButton("Item 3");
	detailImage3RB.setActionCommand("3");
	detailImage4RB = new JRadioButton("Item 4");
	detailImage4RB.setActionCommand("4");
	
	detailImageBGLabel = new JLabel("Select Main Media Item: ");
	detailImageBG = new ButtonGroup();
	detailImageBG.add(detailImage1RB);
	detailImageBG.add(detailImage2RB);
	detailImageBG.add(detailImage3RB);
	detailImageBG.add(detailImage4RB);
	
	gbc = new GridBagConstraints(0, 9, 3, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailImageBGLabel, gbc);
	
	gbc = new GridBagConstraints(3, 9, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailImage1RB, gbc);
	
	gbc = new GridBagConstraints(4, 9, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailImage2RB, gbc);
	
	gbc = new GridBagConstraints(5, 9, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailImage3RB, gbc);
	
	gbc = new GridBagConstraints(6, 9, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(detailImage4RB, gbc);
	
	
	gbc = new GridBagConstraints(0, 10, 2, 1, 0.0, 1.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,5,0,0), 0, 0);
	gbl.setConstraints(descriptionLabel, gbc);

	gbc = new GridBagConstraints(2, 10, 12, 1, 0.0, 1.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0,0,0,0), 0, 0);
	gbc.fill = GridBagConstraints.BOTH;
	gbl.setConstraints(descriptionSP, gbc);   
    
    
	//add components to detail panel
	detailDataPanel.add(detailTabLabel);
	detailDataPanel.add(detailTabHelpButton);
	detailDataPanel.add(detailText1Label);
	detailDataPanel.add(detailText1TF);
	detailDataPanel.add(detailText2Label);
	detailDataPanel.add(detailText2TF);
	detailDataPanel.add(detailText3Label);
	detailDataPanel.add(detailText3TF);
	detailDataPanel.add(detailText4Label);
	detailDataPanel.add(detailText4TF);
	detailDataPanel.add(detailImage1Label);
	detailDataPanel.add(detailImage1TF);
	detailDataPanel.add(detailImage1Button);
	detailDataPanel.add(detailImage2Label);
	detailDataPanel.add(detailImage2TF);
	detailDataPanel.add(detailImage2Button);
	detailDataPanel.add(detailImage3Label);
	detailDataPanel.add(detailImage3TF);
	detailDataPanel.add(detailImage3Button);
	detailDataPanel.add(detailImage4Label);
	detailDataPanel.add(detailImage4TF);
	detailDataPanel.add(detailImage4Button);
	detailDataPanel.add(descriptionLabel);
	detailDataPanel.add(descriptionSP);
	detailDataPanel.add(detailImageBGLabel);
	detailDataPanel.add(detailImage1RB);
	detailDataPanel.add(detailImage2RB);
	detailDataPanel.add(detailImage3RB);
	detailDataPanel.add(detailImage4RB);
    
	//sets button area - first, previous, next, last, delete, update, add
	buttonPanel = new JPanel();
	buttonPanel.setLayout(gbl);
    
	addButton = new JButton("Add Event");
	addButton.addActionListener(new AddNewEvent(this));
	addButton.createToolTip();
	addButton.setToolTipText("Adds the new Event to the table");

	deleteButton = new JButton("Delete Event");
	deleteButton.addActionListener(new DeleteEvent(this, jt));
	deleteButton.createToolTip();
	deleteButton.setToolTipText("Deletes the Event from the table");
    
	replaceButton = new JButton("Update Event");
	replaceButton.addActionListener(new ReplaceEvent(this, jt));
	replaceButton.createToolTip();
	replaceButton.setToolTipText("Updates the Event in the table with the changes");
    
	prevButton = new JButton("< Previous");
	prevButton.addActionListener(new PreviousEvent(this, jt));
	prevButton.createToolTip();
	prevButton.setToolTipText("Goes to the previous Event in the table");
        
	nextButton = new JButton("Next >");
	nextButton.addActionListener(new NextEvent(jt, mainTabbedPane));
	nextButton.createToolTip();
	nextButton.setToolTipText("Goes to the next Event in the table");
        
	firstButton = new JButton("|<< First");
	firstButton.addActionListener(new FirstEvent(jt, mainTabbedPane));
	firstButton.createToolTip();
	firstButton.setToolTipText("Goes to the first Event in the table");    
    
	lastButton = new JButton("Last >>|");
	lastButton.addActionListener(new LastEvent(jt, mainTabbedPane));
	lastButton.createToolTip();
	lastButton.setToolTipText("Goes to the last Event in the table");        


	gbc = new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0);
	gbl.setConstraints(firstButton, gbc);    

	gbc = new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0);
	gbl.setConstraints(prevButton, gbc);    

	gbc = new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0);
	gbl.setConstraints(nextButton, gbc);    

	gbc = new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0);
	gbl.setConstraints(lastButton, gbc);    

	gbc = new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0);
	gbl.setConstraints(deleteButton, gbc);    

	gbc = new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0);
	gbl.setConstraints(replaceButton, gbc);    

	gbc = new GridBagConstraints(2, 1, 2, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1,1,1,1), 0, 0);
	gbl.setConstraints(addButton, gbc);    

	//add buttons to panel
	buttonPanel.add(firstButton);
	buttonPanel.add(prevButton);
	buttonPanel.add(nextButton);
	buttonPanel.add(lastButton);
	buttonPanel.add(replaceButton);
	buttonPanel.add(addButton);
	buttonPanel.add(deleteButton);
        
	//add the tabs and then associate a panel with the labels (tabs)
	dataTabbedPane = new JTabbedPane();
	dataTabbedPane.addTab("General",generalDataPanel);
	dataTabbedPane.addTab("Detail",detailDataPanel);
	dataTabbedPane.setAutoscrolls(true);
	//see screen shot for a detailed understanding of containers
	dataEditPanel = new JPanel();
	dataEditPanel.setLayout(new BoxLayout(dataEditPanel, BoxLayout.Y_AXIS));
	dataEditPanel.add(dataTabbedPane);
	dataEditPanel.add(buttonPanel);
	dataEditPanel.setAutoscrolls(true);
   
	containerPanel = new JPanel();  //sets window a layout
    
	tableScroll = new JScrollPane(jt, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

	//adds the table and the form area to the parent panel
    
	/*
	  containerPanel.setLayout(new GridLayout(2, 1));
	  containerPanel.add(tableScroll); 
	  containerPanel.add(dataEditPanel);
	*/


	//tableScroll.setPreferredSize(new Dimension(540, 160));
	//tableScroll.setMinimumSize(new Dimension(540, 80));

	//dataEditPanel.setPreferredSize(new Dimension(520, 420));
	//dataEditPanel.setMinimumSize(new Dimension(520, 420));
	//dataEditPanel.setMaximumSize(new Dimension(520, 420));




	containerPanel.setLayout(new BoxLayout(containerPanel, BoxLayout.Y_AXIS));
	containerPanel.add(tableScroll); 
	containerPanel.add(dataEditPanel);



	//up to this components for data tab are complete

	//create config panel
	configPanel = new JPanel();
	configPanel.setLayout(gbl);
    
	//setting labels
	configTabLabel = new JLabel("<html><b>Enter information for the timeline interface.</b></html>");
	configTitleLabel = new JLabel("Timeline Title:");
	configStartLabel = new JLabel("Timeline Start:");
	configStopLabel = new JLabel("Timeline Stop:");
	configUnitsLabel = new JLabel("Units:");
	configSpacingLabel = new JLabel("Gradation:");
	configBand1TitleLabel = new JLabel("Band 1 Title:");
	configBand1ColorLabel = new JLabel("Band 1 Color:");
	configBand2TitleLabel = new JLabel("Band 2 Title:");
	configBand2ColorLabel = new JLabel("Band 2 Color:");
	configBand3TitleLabel = new JLabel("Band 3 Title:");
	configBand3ColorLabel = new JLabel("Band 3 Color:");
	configBand4TitleLabel = new JLabel("Band 4 Title:");
	configBand4ColorLabel = new JLabel("Band 4 Color:");
	configBand5TitleLabel = new JLabel("Band 5 Title:");
	configBand5ColorLabel = new JLabel("Band 5 Color:");
	configDetail1TitleLabel = new JLabel("Detail 1 Title:");
	configDetail2TitleLabel = new JLabel("Detail 2 Title:");
	configDetail3TitleLabel = new JLabel("Detail 3 Title:");
	configDetail4TitleLabel = new JLabel("Detail 4 Title:");
	configBGColorLabel = new JLabel("Main Color:");
	configBorderColorLabel = new JLabel("Border Color:");
	configTextColorLabel = new JLabel("Text Color:");
	configTextFontLabel = new JLabel("Text Font:");
	configCopyrightLabel = new JLabel("Credits:");

	//size of text fields
	configTitleTF = new JTextField(150);
	configStartTF = new JTextField(40);
	configStopTF = new JTextField(40);
	configUnitsTF = new JTextField(40);
	configSpacingTF = new JTextField(40);
	configBand1TitleTF = new JTextField(30);
	configBand2TitleTF = new JTextField(30);
	configBand3TitleTF = new JTextField(30);
	configBand4TitleTF = new JTextField(30);
	configBand5TitleTF = new JTextField(30);
	configDetail1TitleTF = new JTextField(30);
	configDetail2TitleTF = new JTextField(30);
	configDetail3TitleTF = new JTextField(30);
	configDetail4TitleTF = new JTextField(30);
	
    
	//create buttons
	//config tab help button
	configTabHelpButton = new JButton("<html><b>Help?</b></html>");
	configTabHelpButton.setForeground(Color.red);
	configTabHelpButton.addActionListener(new GenericHelp());
	//create color picker buttons    
	configBand1ColorButton = new JButton("Band 1");
	configBand1ColorButton.setBackground(Color.lightGray);
	configBand1ColorButton.setForeground(Color.lightGray);
	prepareColorButton(configBand1ColorButton);
	configBand1ColorButton.setToolTipText("Click here to set the Band 1 color");
    
	configBand2ColorButton = new JButton("Band 2");
	configBand2ColorButton.setBackground(Color.cyan);
	configBand2ColorButton.setForeground(Color.cyan);
	prepareColorButton(configBand2ColorButton);
	configBand2ColorButton.setToolTipText("Click here to set the Band 2 color");
     
	configBand3ColorButton = new JButton("Band 3");
	configBand3ColorButton.setBackground(Color.gray);
	configBand3ColorButton.setForeground(Color.gray);
	prepareColorButton(configBand3ColorButton);
	configBand3ColorButton.setToolTipText("Click here to set the Band 3 color");
 
	configBand4ColorButton = new JButton("Band 4");
	configBand4ColorButton.setBackground(Color.blue);
	configBand4ColorButton.setForeground(Color.blue);
	prepareColorButton(configBand4ColorButton);
	configBand4ColorButton.setToolTipText("Click here to set the Band 4 color");
   
	configBand5ColorButton = new JButton("Band 5");
	configBand5ColorButton.setBackground(Color.white);
	configBand5ColorButton.setForeground(Color.white);
	prepareColorButton(configBand5ColorButton);
	configBand5ColorButton.setToolTipText("Click here to set the Band 5 color");
     
	configBGColorButton = new JButton("Background");
	configBGColorButton.setBackground(Color.orange);
	configBGColorButton.setForeground(Color.orange);
	prepareColorButton(configBGColorButton);
	configBGColorButton.setToolTipText("Click here to set the main background color");
 
	configBorderColorButton = new JButton("Border");
	configBorderColorButton.setBackground(Color.pink);
	configBorderColorButton.setForeground(Color.pink);
	prepareColorButton(configBorderColorButton);
	configBorderColorButton.setToolTipText("Click here to set the border color");
 
	configTextColorButton = new JButton("Text");
	configTextColorButton.setBackground(Color.green);
	configTextColorButton.setForeground(Color.green);
	prepareColorButton(configTextColorButton);
	configTextColorButton.setToolTipText("Click here to set the text color");
    
     
    
	configCopyrightTF = new JTextField(30);
    
	//names for font choice
	String[] fontnames = {"Serif","San-Serif"};
	configTextFontCB = new JComboBox(fontnames);
    
	Insets config_insets = new Insets(5,10,0,5);  //default insets for layout
    
    
	//again layout by row
	//extra line break indicates new row
	gbc = new GridBagConstraints(0, 0, 7, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5,10,10,0), 0, 0);
	gbl.setConstraints(configTabLabel, gbc);
    
	gbc = new GridBagConstraints(7, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(5,30,10,0), 0, 0);
	gbl.setConstraints(configTabHelpButton, gbc);


	gbc = new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configTitleLabel, gbc);
 
	gbc = new GridBagConstraints(1, 1, 7, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configTitleTF, gbc);
    

	gbc = new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configStartLabel, gbc);

	gbc = new GridBagConstraints(1, 2, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 120, 0);
	gbl.setConstraints(configStartTF, gbc);

	gbc = new GridBagConstraints(4, 2, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configStopLabel, gbc);

	gbc = new GridBagConstraints(5, 2, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 120, 0);
	gbl.setConstraints(configStopTF, gbc);


	gbc = new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configUnitsLabel, gbc);

	gbc = new GridBagConstraints(1, 3, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configUnitsTF, gbc);

	gbc = new GridBagConstraints(4, 3, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configSpacingLabel, gbc);

	gbc = new GridBagConstraints(5, 3, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configSpacingTF, gbc);


	gbc = new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBGColorLabel, gbc);

	gbc = new GridBagConstraints(1, 4, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBGColorButton, gbc);

	gbc = new GridBagConstraints(4, 4, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBorderColorLabel, gbc);

	gbc = new GridBagConstraints(5, 4, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBorderColorButton, gbc);


	gbc = new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configTextColorLabel, gbc);

	gbc = new GridBagConstraints(1, 5, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configTextColorButton, gbc);

	gbc = new GridBagConstraints(4, 5, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configTextFontLabel, gbc);

	gbc = new GridBagConstraints(5, 5, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configTextFontCB, gbc);
    

	gbc = new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand1TitleLabel, gbc);

	gbc = new GridBagConstraints(1, 6, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand1TitleTF, gbc);

	gbc = new GridBagConstraints(4, 6, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand1ColorLabel, gbc);

	gbc = new GridBagConstraints(5, 6, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand1ColorButton, gbc);


	gbc = new GridBagConstraints(0, 7, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand2TitleLabel, gbc);

	gbc = new GridBagConstraints(1, 7, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand2TitleTF, gbc);

	gbc = new GridBagConstraints(4, 7, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand2ColorLabel, gbc);

	gbc = new GridBagConstraints(5, 7, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand2ColorButton, gbc);


	gbc = new GridBagConstraints(0, 8, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand3TitleLabel, gbc);

	gbc = new GridBagConstraints(1, 8, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand3TitleTF, gbc);

	gbc = new GridBagConstraints(4, 8, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand3ColorLabel, gbc);

	gbc = new GridBagConstraints(5, 8, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand3ColorButton, gbc);


	gbc = new GridBagConstraints(0, 9, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand4TitleLabel, gbc);

	gbc = new GridBagConstraints(1, 9, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand4TitleTF, gbc);

	gbc = new GridBagConstraints(4, 9, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand4ColorLabel, gbc);

	gbc = new GridBagConstraints(5, 9, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand4ColorButton, gbc);


	gbc = new GridBagConstraints(0, 10, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand5TitleLabel, gbc);

	gbc = new GridBagConstraints(1, 10, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand5TitleTF, gbc);

	gbc = new GridBagConstraints(4, 10, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configBand5ColorLabel, gbc);

	gbc = new GridBagConstraints(5, 10, 3, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configBand5ColorButton, gbc);


	gbc = new GridBagConstraints(0, 11, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configDetail1TitleLabel, gbc);

	gbc = new GridBagConstraints(1, 11, 7, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configDetail1TitleTF, gbc);


	gbc = new GridBagConstraints(0, 12, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configDetail2TitleLabel, gbc);

	gbc = new GridBagConstraints(1, 12, 7, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configDetail2TitleTF, gbc);


	gbc = new GridBagConstraints(0, 13, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configDetail3TitleLabel, gbc);

	gbc = new GridBagConstraints(1, 13, 7, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configDetail3TitleTF, gbc);


	gbc = new GridBagConstraints(0, 14, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, config_insets, 0, 0);
	gbl.setConstraints(configDetail4TitleLabel, gbc);

	gbc = new GridBagConstraints(1, 14, 7, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, config_insets, 0, 0);
	gbl.setConstraints(configDetail4TitleTF, gbc);


	gbc = new GridBagConstraints(0, 15, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5,10,30,5), 0, 0);
	gbl.setConstraints(configCopyrightLabel, gbc);

	gbc = new GridBagConstraints(1, 15, 7, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5,10,30,5), 0, 0);
	gbl.setConstraints(configCopyrightTF, gbc);

	//adding everything to config panel
	configPanel.add(configTabLabel);
	configPanel.add(configTabHelpButton);
	configPanel.add(configTitleLabel);
	configPanel.add(configTitleTF);
	configPanel.add(configStartLabel);
	configPanel.add(configStartTF);
	configPanel.add(configStopLabel);
	configPanel.add(configStopTF);
	configPanel.add(configUnitsLabel);
	configPanel.add(configUnitsTF);
	configPanel.add(configSpacingLabel);
	configPanel.add(configSpacingTF);
	configPanel.add(configBand1TitleLabel);
	configPanel.add(configBand1TitleTF);
	configPanel.add(configBand1ColorLabel);
	configPanel.add(configBand1ColorButton);
	configPanel.add(configBand2TitleLabel);
	configPanel.add(configBand2TitleTF);
	configPanel.add(configBand2ColorLabel);
	configPanel.add(configBand2ColorButton);
	configPanel.add(configBand3TitleLabel);
	configPanel.add(configBand3TitleTF);
	configPanel.add(configBand3ColorLabel);
	configPanel.add(configBand3ColorButton);
	configPanel.add(configBand4TitleLabel);
	configPanel.add(configBand4TitleTF);
	configPanel.add(configBand4ColorLabel);
	configPanel.add(configBand4ColorButton);
	configPanel.add(configBand5TitleLabel);
	configPanel.add(configBand5TitleTF);
	configPanel.add(configBand5ColorLabel);
	configPanel.add(configBand5ColorButton);
	configPanel.add(configDetail1TitleLabel);
	configPanel.add(configDetail1TitleTF);
	configPanel.add(configDetail2TitleLabel);
	configPanel.add(configDetail2TitleTF);
	configPanel.add(configDetail3TitleLabel);
	configPanel.add(configDetail3TitleTF);
	configPanel.add(configDetail4TitleLabel);
	configPanel.add(configDetail4TitleTF);
	configPanel.add(configBGColorLabel);
	configPanel.add(configBGColorButton);
	configPanel.add(configBorderColorLabel);
	configPanel.add(configBorderColorButton);
	configPanel.add(configTextColorLabel);
	configPanel.add(configTextColorButton);
	configPanel.add(configTextFontLabel);
	configPanel.add(configTextFontCB);
	configPanel.add(configCopyrightTF);
	configPanel.add(configCopyrightLabel);
    
	//below adds the complete (component filled) panels to tabs
	mainTabbedPane = new JTabbedPane();
	mainTabbedPane.addChangeListener(new bandListener());
	mainTabbedPane.addTab("Events",containerPanel);
	mainTabbedPane.addTab("Interface Configuration",configPanel);
    
	//mainTabbedPane.setSelectedIndex(1);
	//adds the complete tabbed pane to program window, and then adds
	getContentPane().add(jmb, BorderLayout.NORTH);    
	getContentPane().add(mainTabbedPane); 
	//need event listener for this to work
	//getContentPane().setMinimumSize(new Dimension(480, 580));

	//System.out.println(UIManager.getSystemLookAndFeelClassName());

	addWindowListener(new WinClosing(this));
	//	setBounds(70, 0, 550, 589);  //this sets the position of the Java window within the desktop (top,left,width, height)

        //int betterheight=getNewHeight(addButton,589); //589 is the default height.
        //int betterheight=getNewHeight(containerPanel,589);

	//containerPanel.setBorder(new EmptyBorder(10,10,10,10));
	//mainTabbedPane.setBorder(new EmptyBorder(10,10,10,10));
	//buttonPanel.setBorder(new EmptyBorder(0,0,30,0));

	//containerPanel.setBounds(0,0,540,500);

	int betterheight=589+150;


	//LM
	//apple.laf.AquaLookAndFeel
	if(UIManager.getSystemLookAndFeelClassName().indexOf("Aqua")!= -1) {
		betterheight=680+150;
	}

        setBounds(70, 0, 550, betterheight);  //this sets the position of the Java window within the desktop (top,left,width, height)
	setTitle(title); //sets window title
	setVisible(true);  //must be used to display
	setResizable(true); //disables resizing of window
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    }
 

    //  //this is where the action is
    //  //control for no detail check box
    //  public void itemStateChanged(ItemEvent ie) {
    //  	JCheckBox jtb = (JCheckBox) ie.getItemSelectable();
    //  	
    //  	if(jtb.isSelected()){
    //  		detailText1TF.setEditable(false);
    //  		detailText2TF.setEditable(false);
    //  		detailText3TF.setEditable(false);
    //  		detailText4TF.setEditable(false);
    //  		detailImageTF.setEditable(false);
    //  		descriptionTA.setEditable(false);
    //  	
    //  	}
    //  	else if(!jtb.isSelected()){
    //  		detailText1TF.setEditable(true);
    //  		detailText2TF.setEditable(true);
    //  		detailText3TF.setEditable(true);
    //  		detailText4TF.setEditable(true);
    //  		detailImageTF.setEditable(true);
    //  		descriptionTA.setEditable(true);
    //  	}
    //}

    /**
     * Prepares JButtons which can popup color palette.
     *
     * @param thisButton JButton passed to function to have color applied
     */
    public void prepareColorButton(JButton thisButton){
   	final JButton myButton = thisButton;
   	myButton.setBorderPainted(true);
    	myButton.setMargin(new Insets(0,0,0,0));
	myButton.createToolTip();
	
    	final JColorChooser colorChooser = new JColorChooser(); //enables the color picker buttons to call color picker
    	ActionListener okListener = new ActionListener() {
            /**
             * Action Listener to change button color when another is selected from palette.
             *
             * @param e ActionEvent passed into function
             */
		public void actionPerformed(ActionEvent e) {
		    myButton.setBackground(colorChooser.getColor());
		    myButton.setForeground(colorChooser.getColor());
		}
	    };
       	final JDialog dialog = JColorChooser.createDialog(myButton,"Pick a Color",true,colorChooser,okListener,null);
        myButton.addActionListener(new ActionListener() {
            /**
             * Action Listener to open color palette.
             *
             *@param e ActionEvent passed into function
             */
		public void actionPerformed(ActionEvent e) {
		    dialog.setLocationRelativeTo(myButton);  //sets the color picker dialog box relative to the button
		    dialog.show(); //shows colorpicker dialog box
		}
	    });  
    } 
    	

    /**
     *figures if current button is off the window.  returns a height that can fit it.
     */
    public int getNewHeight(Component c,int curheight) {
	int endpos;

	endpos=c.getHeight() + c.getX();
	return (endpos > curheight) ? (endpos+10) : curheight;
    }

    /**
     * listener to change the band titles in the band combo box to the titles set in the config tab
     */
	class bandListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			setBands();
		}
	}
	// This function is seperate because we call it during initialization when a file is loaded
	public void setBands() {
		String temp;
		
		temp = configBand1TitleTF.getText();
		if (temp.length() == 0) temp = "sub1";
		bandCB.removeItemAt(1);
		bandCB.insertItemAt(temp,1);
		
		temp = configBand2TitleTF.getText();
		if (temp.length() == 0) temp = "sub2";
		bandCB.removeItemAt(2);
		bandCB.insertItemAt(temp,2);

		temp = configBand3TitleTF.getText();
		if (temp.length() == 0) temp = "sub3";
		bandCB.removeItemAt(3);
		bandCB.insertItemAt(temp,3);

		temp = configBand4TitleTF.getText();
		if (temp.length() == 0) temp = "sub4";
		bandCB.removeItemAt(4);
		bandCB.insertItemAt(temp,4);

		temp = configBand5TitleTF.getText();
		if (temp.length() == 0) temp = "sub5";
		bandCB.removeItemAt(5);
		bandCB.insertItemAt(temp,5);
	}

    
    /**
     * Sets UI look and feel, defaults to the what the system look and feel is so that it will work on all platforms
     */
    public static void main(String args[]) {  //main of program
	try {
	    //UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()); 
	    //UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception e) { }
	//SplashScreen ss = new SplashScreen(6000);
	TimelineCreator tb = new TimelineCreator();  //creates the program
    }
}
