package edu.jhu.cer.TimelineCreator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.FileInputStream;
import java.io.IOException;
import org.xml.sax.SAXException;
import javax.swing.table.DefaultTableModel;
import javax.swing.JTextField;
import javax.swing.JTable;
import javax.swing.JPanel;
import javax.swing.table.TableModel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import java.awt.Color;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 * A class to read in Timeline data from a valid Timeline.xml file.
 * After using the SAX parser to read in the data, and verify that is is valid XML, this class
 * inserts the data into the appropriate UI widgets in the main
 * TimelineCreator window.
 */

class TimelineDataReader {
    /**
     * A reference to the main {@link TimelineCreator} object, which gets
     * used when we want to bring up a warning window.
     */
    TimelineCreator creator;

    /**
     * The table used to store all the event data back in the {@link TimelineCreator}.
     */
    JTable table;

    /**
     * The table model associated with the table {@link #table table}. This is used for adding rows.
     */
    TableModel table_model;

    /**
     * Sets up class variables.
     * @param tc The parent {@link TimelineCreator} object
     * @param jt The table storing all event data
     */
    public TimelineDataReader(TimelineCreator tc, JTable jt) {
	creator = tc;
	table = jt;
	table_model = jt.getModel();
    }

    void readData(String filename) {
	// create DocumentBuilder Objects to parse the data
	DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	DocumentBuilder db = null;

	// create the new document builder
	try {
	    db = dbf.newDocumentBuilder();
	} catch (ParserConfigurationException pce) {
	    System.err.println(pce);
	    System.exit(1);
	}
	
	Document doc = null;
	try {
	    FileInputStream fis = new FileInputStream(filename);

	    // try to parse the file
	    doc = db.parse(fis);

	    fis.close();
	} catch (SAXException se) {
	    System.err.println(se.getMessage());
	    
	    // open an error window to alert the user the file they've selected is not an xml file
	    String error_msg = "The file you have attempted to open\n";
	    error_msg += "["+filename+"]\n";
	    error_msg += "is not a valid Timeline XML file.\nPlease choose another.";
				
	    // maybe use getTopLevelAncestor?
	    JOptionPane.showMessageDialog(creator, error_msg, "XML Error", JOptionPane.ERROR_MESSAGE); 
	    return;
	} catch (IOException ioe) {
	    System.err.println(ioe);
	    System.exit(1);
	}

	// get count of data points
	NodeList elements = doc.getElementsByTagName("eventCount");
	int eventCount = Integer.parseInt(elements.item(0).getFirstChild().getNodeValue());
	
	//Begin for loop to create space to add data in based on number of data points (i)
	((DefaultTableModel)table_model).setRowCount(0);

	// what is this doing?
	Object emptydata[] = {};  	
	for(int i = 0; i<= eventCount; i++){
	    ((DefaultTableModel)table_model).addRow(emptydata);
	}
			
	//passing in current document, config text field, and then the string in the XML document
	setConfigTF(doc,creator.configTitleTF,"title");
	setConfigTF(doc,creator.configStartTF ,"start");
	setConfigTF(doc,creator.configStopTF ,"stop");
	setConfigTF(doc,creator.configUnitsTF,"units");
	setConfigTF(doc,creator.configSpacingTF ,"gradationspacing");
	setConfigButton(doc,creator.configBand1ColorButton ,"band1color");
	setConfigTF(doc,creator.configBand1TitleTF ,"band1title");
	setConfigButton(doc,creator.configBand2ColorButton ,"band2color");
	setConfigTF(doc,creator.configBand2TitleTF ,"band2title");
	setConfigButton(doc,creator.configBand3ColorButton ,"band3color");
	setConfigTF(doc,creator.configBand3TitleTF ,"band3title");
	setConfigButton(doc,creator.configBand4ColorButton ,"band4color");
	setConfigTF(doc,creator.configBand4TitleTF ,"band4title");
	setConfigButton(doc,creator.configBand5ColorButton ,"band5color");
	setConfigTF(doc,creator.configBand5TitleTF ,"band5title");
	setConfigTF(doc,creator.configDetail1TitleTF ,"detail1title");
	setConfigTF(doc,creator.configDetail2TitleTF ,"detail2title");
	setConfigTF(doc,creator.configDetail3TitleTF ,"detail3title");
	setConfigTF(doc,creator.configDetail4TitleTF ,"detail4title");
	setConfigButton(doc,creator.configBGColorButton ,"backgroundcolor");
	setConfigButton(doc,creator.configBorderColorButton ,"bordercolor");
	setConfigButton(doc,creator.configTextColorButton ,"textcolor");
	setConfigCB(doc,creator.configTextFontCB ,"textfont");
	setConfigTF(doc,creator.configCopyrightTF ,"copyright");
	//document, name in XML, column number
	setData(doc,"hide",15);			
	setData(doc,"title1",0);
	setData(doc,"iconid",4);
	setData(doc,"startdata",1);
	setData(doc,"enddata",2);
	setData(doc,"band",3);
	setData(doc,"hidedetail",5);
	setDataSpecial(doc,"detailimagefile1",10);
	setData(doc,"detailimagefile2",11);
	setData(doc,"detailimagefile3",12);
	setData(doc,"detailimagefile4",13);
	setData(doc,"detailtext1",6);
	setData(doc,"detailtext2",7);
	setData(doc,"detailtext3",8);
	setData(doc,"detailtext4",9);
	setData(doc,"description",14);
	setDataSpecial(doc,"imagefile",16);
		
	// apparently, these are not used
	// IsTableEdited.tableChanged = false;
	// creator.CurrentFileName = justname;
    }

    /**
     * Function which sets TextField values in config panel.
     *
     * @param myDoc document to be opened
     *        fieldName JTextField to be modified
     *        xmlName XML file to be read from
     */
    public void setConfigTF(Document myDoc, JTextField fieldName, String xmlName){
	NodeList elements = myDoc.getElementsByTagName(xmlName);
	if ((elements.item(0).hasChildNodes()) && (elements.item(0).getFirstChild().getNodeValue() != null)) {
	    fieldName.setText(elements.item(0).getFirstChild().getNodeValue());
	}
	else fieldName.setText("");
    }

    /**
     * Function which sets ComboBox values in config panel.
     *
     * @param myDoc document to be opened
     *        fieldName JComboBox to be modified
     *        xmlName XML file to be read from
     */
    public void setConfigCB(Document myDoc, JComboBox fieldName, String xmlName){
	NodeList elements = myDoc.getElementsByTagName(xmlName);
	if (elements.item(0).hasChildNodes()) {
	    fieldName.setSelectedItem(elements.item(0).getFirstChild().getNodeValue());
	}
    }

    /**
     * Function which sets Button values in config panel.
     *
     * @param myDoc document to be opened
     *        fieldName JButton to be modified
     *        xmlName XML file to be read from
     */
    public void setConfigButton(Document myDoc, JButton fieldName, String xmlName){
	NodeList elements = myDoc.getElementsByTagName(xmlName);
	if (elements.item(0).hasChildNodes()) {
	    String RGBval = elements.item(0).getFirstChild().getNodeValue();
  		
	    String Rval = RGBval.substring(0,2);
	    String Gval = RGBval.substring(2,4);
	    String Bval = RGBval.substring(4,6);
  		
	    int Rint = Integer.parseInt(Rval,16);
	    int Gint = Integer.parseInt(Gval,16);
	    int Bint = Integer.parseInt(Bval,16);
  		
	    Color buttonColor = new Color(Rint,Gint,Bint);
	    fieldName.setBackground(buttonColor);
	    fieldName.setForeground(buttonColor);
	}	
    }		

    /**
     * Function that sets values in data panel
     *
     * @param myDoc document to be opened
     *        xmlName XML file to be read from
     *        colNum column # to be modified
     */
    public void setData (Document myDoc, String xmlName, int colNum){
	NodeList elements = myDoc.getElementsByTagName(xmlName);
	if (elements.getLength() == 0 || elements == null ){
		for(int i = 0; i < myDoc.getElementsByTagName("band").getLength(); i++ )
			table.setValueAt("",i,colNum);	
	}
	else{
		for (int i = 0; i < elements.getLength(); i++) {
		
			if((elements.item(i).hasChildNodes())&& !(elements.item(i).getFirstChild().getNodeValue().equals("null"))){
					table.setValueAt(elements.item(i).getFirstChild().getNodeValue(),i,colNum);
			}
			else 
				table.setValueAt("",i,colNum);
		}	
	}
    }
    /**
     * Makes sure we are backwards compatible in reading in files
     */
    public void setDataSpecial (Document myDoc, String xmlName, int colNum){
    	
    	
    	NodeList elements = myDoc.getElementsByTagName(xmlName);
    	//reads from XLM tag detailimagefile instead of detailimagefile1 if latter does not exist
    	if( elements.getLength() == 0 && xmlName == "detailimagefile1")
    	{
    		xmlName = "detailimagefile";
    		elements = myDoc.getElementsByTagName(xmlName);
    	}
    	for (int i = 0; i < elements.getLength(); i++) {
    	    if((elements.item(i).hasChildNodes())&& !(elements.item(i).getFirstChild().getNodeValue().equals("null"))){
    		table.setValueAt(elements.item(i).getFirstChild().getNodeValue(),i,colNum);
    	    }
    	    else 
				table.setValueAt("",i,colNum);
    	}	
    	
    	if( xmlName == ("imagefile") )
    	{
    		NodeList list0 = myDoc.getElementsByTagName("detailimagefile");
    		NodeList list1 = myDoc.getElementsByTagName("detailimagefile1");
    		NodeList list2 = myDoc.getElementsByTagName("detailimagefile2");
    		NodeList list3 = myDoc.getElementsByTagName("detailimagefile3");
    		NodeList list4 = myDoc.getElementsByTagName("detailimagefile4");
    		
    		
    		for (int i = 0; i < elements.getLength(); i++) {
    			if (elements.item(i).hasChildNodes())
    			{
	    			if( list1.getLength()!=0 && (elements.item(i).getFirstChild().getNodeValue().equals(list1.item(i).getFirstChild().getNodeValue())))
	    				table.setValueAt("1",i,colNum);
	    			else if( list2.getLength()!=0 && (elements.item(i).getFirstChild().getNodeValue().equals(list2.item(i).getFirstChild().getNodeValue())))
	    				table.setValueAt("2",i,colNum);
	    			else if( list3.getLength()!=0 && (elements.item(i).getFirstChild().getNodeValue().equals(list3.item(i).getFirstChild().getNodeValue())))
	    				table.setValueAt("3",i,colNum);
	    			else if( list4.getLength()!=0 && (elements.item(i).getFirstChild().getNodeValue().equals(list4.item(i).getFirstChild().getNodeValue())))
	    				table.setValueAt("4",i,colNum);
	    			else 
	    				table.setValueAt("1",i,colNum);
    			}
				else 
	    		{
	    			table.setValueAt("1",i,colNum);System.out.println("1");
				}
						
    		
    		
    		}
    	}		
    }
}
