package edu.jhu.cer.TimelineCreator;
import edu.jhu.cer.CollisionGroup.PointGrouper;
import edu.jhu.cer.CollisionGroup.Group;
import edu.jhu.cer.CollisionGroup.TLPoint;

import javax.swing.table.TableModel;
import javax.swing.JTable;
import javax.swing.JPanel;
import javax.swing.JButton;
import java.io.FileOutputStream;
import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Vector;
import java.util.Arrays;
import java.lang.reflect.Array;
import java.lang.String;

class TimelineDataWriter {
    /**
     * A reference to the main {@link TimelineCreator} object, which gets
     * used when we want to bring up a warning window.
     */
    TimelineCreator creator;

    /**
     * The table model associated with the table {@link #jt jt}. This is used for adding rows.
     */
    TableModel table_model;

    HashMap sameDate;
    Object[] sameDatesArray;

    /**
     * The table used to store all the event data back in the {@link TimelineCreator}
     **/
    JTable jt;

    /**
     * Sets up class variables.
     * @param tc The parent {@link TimelineCreator} object
     */
    public TimelineDataWriter(TimelineCreator tc) {
	creator = tc;
	jt = creator.jt;
	table_model = jt.getModel();
    }

    /**
     * A wrapper function around modAmps for instances where the object
     * might be null. For instance, something like jt.toString(), where jt is null.
     * @param obj {@link Object} which we want to turn into a string with ampersands fixed
     */
    protected String modAmpsOnObject(Object obj) {
	String str = "";

	if(obj != null) {
	    str = obj.toString();
	}

	return modAmps(str);
    }

    /**
     * Given a string, searches for '&', and replaces it with '&amp;'
     * Also calls the modBrackets method
     * @param txt The initial string {@link String} object
     */
    protected String modAmps(String txt){
	/*String[] tmpArray = new String[0];
	tmpArray = txt.split("&");
	if(tmpArray.length == 1) return modBrackets(txt);
	txt = "";
	for(int i = 0; i < tmpArray.length; i++) {
	    if(i+1 == tmpArray.length) {
		txt += tmpArray[i];
		break;
	    }
	    if(tmpArray[i+1].indexOf("amp;") == 0)
		txt += tmpArray[i] + "&";// + tmpArray[i+1];
	    else {
		txt += tmpArray[i] + "&amp;";
	    }
	}*/
	txt = txt.replaceAll("&","&amp;");
	txt = modBrackets(txt);
	return txt;
    }
    
    protected String modBrackets( String txt )
    {
    	txt = txt.replaceAll("<","&lt;");
    	txt = txt.replaceAll(">"," &gt;");
    	return txt;
    }

    public void writeData(String filename){
	try {
	    FileOutputStream fos = new FileOutputStream(filename);
	    BufferedOutputStream bos = new BufferedOutputStream(fos);
	    DataOutputStream dos = new DataOutputStream(bos);

      /* Deprecated.  Characters are no longer escaped when viewed from the interface     
        
       //Need to go through text fields and replace '&' with '&amp;'
	    //GO THROUGH INDIVIDUALLY AND CALL EACH ONE
	    creator.configTitleTF.setText(modAmps(creator.configTitleTF.getText()));
	    creator.configStartTF.setText(modAmps(creator.configStartTF.getText()));
	    creator.configStopTF.setText(modAmps(creator.configStopTF.getText()));
	    creator.configUnitsTF.setText(modAmps(creator.configUnitsTF.getText()));
	    creator.configBand1TitleTF.setText(modAmps(creator.configBand1TitleTF.getText()));
	    creator.configBand2TitleTF.setText(modAmps(creator.configBand2TitleTF.getText()));
	    creator.configBand3TitleTF.setText(modAmps(creator.configBand3TitleTF.getText()));
	    creator.configBand4TitleTF.setText(modAmps(creator.configBand4TitleTF.getText()));
	    creator.configBand5TitleTF.setText(modAmps(creator.configBand5TitleTF.getText()));
            creator.configDetail1TitleTF.setText(modAmps(creator.configDetail1TitleTF.getText()));
	    creator.configDetail2TitleTF.setText(modAmps(creator.configDetail2TitleTF.getText()));
	    creator.configDetail3TitleTF.setText(modAmps(creator.configDetail3TitleTF.getText()));
	    creator.configDetail4TitleTF.setText(modAmps(creator.configDetail4TitleTF.getText()));
	    creator.configCopyrightTF.setText(modAmps(creator.configCopyrightTF.getText()));
*/
	    //BufferedWriter bw = new BufferedWriter(new FileWriter(filename));
	    //String newline = System.getProperty("line.separator");
	    //bw buffered writer, writes as data is read
	    dataStreamPrintString(dos, "<?xml version=\"1.0\"?>\n");
	    dataStreamPrintString(dos, "\n\n<timeline>\n\n");
	    dataStreamPrintString(dos, "\t<timelineconfig>\n");
	    dataStreamPrintString(dos, "\t\t<title>" + modAmps(creator.configTitleTF.getText()) + "</title>\n");
	    dataStreamPrintString(dos, "\t\t<start>" + modAmps(creator.configStartTF.getText())+ "</start>\n");
	    dataStreamPrintString(dos, "\t\t<stop>" + modAmps(creator.configStopTF.getText()) + "</stop>\n");
	    dataStreamPrintString(dos, "\t\t<units>" + modAmps(creator.configUnitsTF.getText()) + "</units>\n");
	    dataStreamPrintString(dos, "\t\t<gradationspacing>" + modAmps(creator.configSpacingTF.getText())+ "</gradationspacing>\n");
	    dataStreamPrintString(dos, "\t\t<band1color>" + modAmps(returnRGB(creator.configBand1ColorButton)) + "</band1color>\n");
	    dataStreamPrintString(dos, "\t\t<band1title>" + modAmps(creator.configBand1TitleTF.getText()) + "</band1title>\n");
	    dataStreamPrintString(dos, "\t\t<band2color>" + modAmps(returnRGB(creator.configBand2ColorButton))+ "</band2color>\n");
	    dataStreamPrintString(dos, "\t\t<band2title>" + modAmps(creator.configBand2TitleTF.getText()) + "</band2title>\n");
	    dataStreamPrintString(dos, "\t\t<band3color>" + modAmps(returnRGB(creator.configBand3ColorButton)) + "</band3color>\n");
	    dataStreamPrintString(dos, "\t\t<band3title>" + modAmps(creator.configBand3TitleTF.getText()) + "</band3title>\n");
	    dataStreamPrintString(dos, "\t\t<band4color>" + modAmps(returnRGB(creator.configBand4ColorButton)) + "</band4color>\n");
	    dataStreamPrintString(dos, "\t\t<band4title>" + modAmps(creator.configBand4TitleTF.getText()) + "</band4title>\n");
	    dataStreamPrintString(dos, "\t\t<band5color>" + modAmps(returnRGB(creator.configBand5ColorButton)) + "</band5color>\n");
	    dataStreamPrintString(dos, "\t\t<band5title>" + modAmps(creator.configBand5TitleTF.getText()) + "</band5title>\n");
	    dataStreamPrintString(dos, "\t\t<detail1title>" + modAmps(creator.configDetail1TitleTF.getText()) + "</detail1title>\n");
	    dataStreamPrintString(dos, "\t\t<detail2title>" + modAmps(creator.configDetail2TitleTF.getText()) + "</detail2title>\n");
	    dataStreamPrintString(dos, "\t\t<detail3title>" + modAmps(creator.configDetail3TitleTF.getText()) + "</detail3title>\n");
	    dataStreamPrintString(dos, "\t\t<detail4title>" + modAmps(creator.configDetail4TitleTF.getText()) + "</detail4title>\n");
	    dataStreamPrintString(dos, "\t\t<backgroundcolor>" + modAmps(returnRGB(creator.configBGColorButton)) + "</backgroundcolor>\n");
	    dataStreamPrintString(dos, "\t\t<bordercolor>" + modAmps(returnRGB(creator.configBorderColorButton)) + "</bordercolor>\n");
	    dataStreamPrintString(dos, "\t\t<textcolor>" + modAmps(returnRGB(creator.configTextColorButton)) + "</textcolor>\n");
	    dataStreamPrintString(dos, "\t\t<textfont>" + modAmpsOnObject(creator.configTextFontCB.getSelectedItem()) + "</textfont>\n");
	    dataStreamPrintString(dos, "\t\t<copyright>" + modAmps(creator.configCopyrightTF.getText()) + "</copyright>\n");
	    dataStreamPrintString(dos, "\t</timelineconfig>\n\n");

	    //uses for loop to insure that data is written to the file
	    int rows = jt.getRowCount() - 1;
	    int cols = jt.getColumnCount() - 1 ;
	    // /t are tabs for presentation purposes
	    // /n are new lines for presentation purposes
	    dataStreamPrintString(dos, "\t<timelinedata>\n");

	    //
	    //           	for(int i = 0;i <= rows; i++) {
	    //               		dataStreamPrintString(dos, "\t\t<event id = \"" + i + "\">\n");
	    //               		dataStreamPrintString(dos, "\t\t\t<hide>" + jt.getValueAt(i,12) + "</hide>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t<title1>" + jt.getValueAt(i,0) + "</title1>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t<iconid>" + jt.getValueAt(i,4) + "</iconid>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t<startdata>" + jt.getValueAt(i,1) + "</startdata>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t<enddata>" + jt.getValueAt(i,2) + "</enddata>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t<band>" + jt.getValueAt(i,3) + "</band>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t<hidedetail>" + jt.getValueAt(i,5) + "</hidedetail>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t<detail>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t\t<detailimagefile>" + jt.getValueAt(i,10) + "</detailimagefile>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t\t<detailtext1>" + jt.getValueAt(i,6) + "</detailtext1>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t\t<detailtext2>" + jt.getValueAt(i,7) + "</detailtext2>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t\t<detailtext3>" + jt.getValueAt(i,8) + "</detailtext3>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t\t<detailtext4>" + jt.getValueAt(i,9) + "</detailtext4>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t\t<description>" + jt.getValueAt(i,11) + "</description>\n");
	    //               		dataStreamPrintString(dos, "\t\t\t</detail>\n");
	    //               		dataStreamPrintString(dos, "\t\t</event>\n");
	    //              	}

	    Vector mainBandDatesVector = new Vector();
	    Vector sub1BandDatesVector = new Vector();
	    Vector sub2BandDatesVector = new Vector();
	    Vector sub3BandDatesVector = new Vector();
	    Vector sub4BandDatesVector = new Vector();
	    Vector sub5BandDatesVector = new Vector();

//System.out.println("Rows: " + rows);

	    for(int i = 0;i <= rows; i++) {
		if(!(jt.getValueAt(i,4).toString().equals("range")) && !(jt.getValueAt(i,15).toString().equalsIgnoreCase("yes"))){
		    if(jt.getValueAt(i,3).toString().equals("main")){
			mainBandDatesVector.add(new Double(jt.getValueAt(i,1).toString()));
		    }
		    else if(jt.getValueAt(i,3).toString().equals("sub1")){
			sub1BandDatesVector.add(new Double(jt.getValueAt(i,1).toString()));
		    }
		    else if(jt.getValueAt(i,3).toString().equals("sub2")){
			sub2BandDatesVector.add(new Double(jt.getValueAt(i,1).toString()));
		    }
		    else if(jt.getValueAt(i,3).toString().equals("sub3")){
			sub3BandDatesVector.add(new Double(jt.getValueAt(i,1).toString()));
		    }
		    else if(jt.getValueAt(i,3).toString().equals("sub4")){
			sub4BandDatesVector.add(new Double(jt.getValueAt(i,1).toString()));
		    }
		    else if(jt.getValueAt(i,3).toString().equals("sub5")){
			sub5BandDatesVector.add(new Double(jt.getValueAt(i,1).toString()));
		    }
		}
	    }

	    String[] bandNames = {"main","sub1","sub2","sub3","sub4","sub5"};
	    for(int k=0;k<=5;k++){
		sameDate = new HashMap();
		HashMap myHM = new HashMap();
		for(int i = 0;i<=rows;i++){
		    if(jt.getValueAt(i,3).toString().equals(bandNames[k]) && !(jt.getValueAt(i,4).toString().equals("range")) && !(jt.getValueAt(i,15).toString().equalsIgnoreCase("yes"))){
			if(!myHM.containsKey(new Double(jt.getValueAt(i,1).toString()))){
			    myHM.put(new Double(jt.getValueAt(i,1).toString()),new Integer(i));
			}
			else{
			    sameDate.put(new Integer(i),new Double(jt.getValueAt(i,1).toString()));
			}
		    }
		}
		sameDatesArray = sameDate.keySet().toArray();

		Object[] datesArray = myHM.keySet().toArray();
		Arrays.sort(datesArray);

		Vector dates = new Vector();
		dates.removeAllElements();
		for(int i = 0;i<datesArray.length;i++){
		    //System.out.println(datesArray[i].toString());
		    if(k == 0 && mainBandDatesVector.contains(new Double(datesArray[i].toString()))){
			dates.add(datesArray[i]);
			for(int xx=0;xx<sameDatesArray.length;xx++){
			    if( Double.parseDouble(sameDate.get(new Integer(sameDatesArray[xx].toString())).toString()) == Double.parseDouble(datesArray[i].toString())){
				dates.add(datesArray[i]);
			    }
			}
			mainBandDatesVector.remove(datesArray[i]);
		    }
		    else if(k == 1 && sub1BandDatesVector.contains(new Double(datesArray[i].toString()))){
			dates.add(datesArray[i]);


			for(int xx=0;xx<sameDatesArray.length;xx++){
			    if( Double.parseDouble(sameDate.get(new Integer(sameDatesArray[xx].toString())).toString()) == Double.parseDouble(datesArray[i].toString())){
				dates.add(datesArray[i]);

			    }
			}
			sub1BandDatesVector.remove(datesArray[i]);
		    }
		    else if(k == 2 && sub2BandDatesVector.contains(new Double(datesArray[i].toString()))){
			dates.add(datesArray[i]);

			for(int xx=0;xx<sameDatesArray.length;xx++){
			    if( Double.parseDouble(sameDate.get(new Integer(sameDatesArray[xx].toString())).toString()) == Double.parseDouble(datesArray[i].toString())){
				dates.add(datesArray[i]);
			    }
			}
			sub2BandDatesVector.remove(datesArray[i]);
		    }
		    else if(k == 3 && sub3BandDatesVector.contains(new Double(datesArray[i].toString()))){
			dates.add(datesArray[i]);

			for(int xx=0;xx<sameDatesArray.length;xx++){
			    if( Double.parseDouble(sameDate.get(new Integer(sameDatesArray[xx].toString())).toString()) == Double.parseDouble(datesArray[i].toString())){
				dates.add(datesArray[i]);
			    }
			}

			sub3BandDatesVector.remove(datesArray[i]);
		    }
		    else if(k == 4 && sub4BandDatesVector.contains(new Double(datesArray[i].toString()))){
			dates.add(datesArray[i]);

			for(int xx=0;xx<sameDatesArray.length;xx++){
			    if( Double.parseDouble(sameDate.get(new Integer(sameDatesArray[xx].toString())).toString()) == Double.parseDouble(datesArray[i].toString())){

				dates.add(datesArray[i]);
			    }
			}
			sub4BandDatesVector.remove(datesArray[i]);
		    }
		    else if(k == 5 && sub5BandDatesVector.contains(new Double(datesArray[i].toString()))){
			dates.add(datesArray[i]);
			for(int xx=0;xx<sameDatesArray.length;xx++){
			    if( Double.parseDouble(sameDate.get(new Integer(sameDatesArray[xx].toString())).toString()) == Double.parseDouble(datesArray[i].toString())){
				dates.add(datesArray[i]);
			    }
			}
			sub5BandDatesVector.remove(datesArray[i]);
		    }
		}

//System.out.println(dates.toString());

		if(k == 0){
		    dataStreamPrintString(dos, "\n\n<main>\n\n");
		}

		else if(k == 1){
		    dataStreamPrintString(dos, "\n\n<sub1>\n\n");
		}
		else if(k == 2){
		    dataStreamPrintString(dos, "\n\n<sub2>\n\n");
		}
		else if(k == 3){
		    dataStreamPrintString(dos, "\n\n<sub3>\n\n");
		}
		else if(k == 4){
		    dataStreamPrintString(dos, "\n\n<sub4>\n\n");
		}

		else if(k == 5){
		    dataStreamPrintString(dos, "\n\n<sub5>\n\n");
		}

		if(dates.size()>0){
		    //get appropriate variables here somehow if not desirable to be hardcoded in:
		    PointGrouper myPG = new PointGrouper(Double.parseDouble(creator.configStartTF.getText()),
							 Double.parseDouble(creator.configStopTF.getText()),
							 11298.75);
		    Group tempGroup = new Group(myPG.start, myPG.end, myPG.max);

//	Having the following values initialized to 0's creates problems when users want events with start date 0
//		    double curDate = 0;
//		    double nextDate = 0;
		    double curDate = Double.NaN;
		    double nextDate = Double.NaN;

		    boolean isFirstTime = true;
		    boolean hasNextDate = false;
		    boolean isGrouping = false;

		    //go through and make groups here
		    curDate = Double.parseDouble(dates.remove(0).toString());         // input curDate here to initialize the loop
		    while(dates.size() > 0 || isFirstTime){
			if(!dates.isEmpty())
			    {
				nextDate = Double.parseDouble(dates.remove(0).toString());
				hasNextDate = true;
			    }
			if(myPG.isOverlapping(curDate, nextDate))
			    {
				isGrouping = true;
				tempGroup.addPoint(new TLPoint(curDate));
			    }
			else
			    {
				if(isGrouping) tempGroup.addPoint(new TLPoint(curDate));
				else myPG.finalStack.push(new TLPoint(curDate));
				isGrouping = false;

				//clear tempGroup since next point isn't overlapping
				if(!tempGroup.isEmpty())
				    {
					myPG.groupStack = tempGroup.export();
					myPG.processGStack();
				    }
			    }
			if(dates.isEmpty()) //this loop is when everything has been removed, so nextDate needs to be added to finalStack or if it is being grouped, grouped and then added
			    {
				if(isGrouping)
				    {
					tempGroup.addPoint(new TLPoint(nextDate));
					myPG.groupStack = tempGroup.export();
					myPG.processGStack();
				    }
				else if(!isFirstTime || hasNextDate) myPG.finalStack.push(new TLPoint(nextDate));
			    }
			curDate = nextDate;                                         // shifts to replace processed date
			isFirstTime = false;
		    }

		    // all dates have been processed and grouped accordingly at this point
		    // finalStack contains all the points and groups in proper order, but simply popping them will give you the points in reverse order
		    // so use finalStack.remove(0) to get the front element one at a time

		    // here's the example of getting the information from the Stack, so you can easily see how to get it all

		    while(!myPG.finalStack.isEmpty()){

//System.out.println("Popping...");

			Object tempO = new Object();
			tempO = myPG.finalStack.remove(0);
			if(tempO instanceof TLPoint)
			    {

//System.out.println("IF");

				//System.out.println("date: " + ((TLPoint)tempO).getDate());
				//System.out.println("HM: " + myHM.get(new Double(((TLPoint)tempO).getDate())).toString());
				//if( ((TLPoint)tempO).getDate() != 0.0){
				int i = Integer.parseInt(myHM.get(new Double(((TLPoint)tempO).getDate())).toString());
				int eventid = i + 1;
				double tempDouble = ((TLPoint)tempO).getDate();
				writeCommonData(dos, i, 0, null, eventid);
				//}
			    }
			//if(tempO instanceof Group)
			else
			    {

//System.out.println("ELSE");

				int mySize = ((Group)tempO).getSize();
				Double[] tempRA = new Double[mySize];
				tempRA = ((Group)tempO).getPointsInGroup();
				double prevDate = 1.3;

//System.out.println("mySize: " + mySize);

				for(int ii = 0;ii<mySize;ii++){

				    Vector temp = new Vector();

				    if(k==0){
					temp = mainBandDatesVector;
				    }
				    else if(k==1){
					temp = sub1BandDatesVector;
				    }
				    else if(k==2){
					temp = sub2BandDatesVector;
				    }
				    else if(k==3){
					temp = sub3BandDatesVector;
				    }
				    else if(k==4){
					temp = sub4BandDatesVector;
				    }
				    else if(k==5){
					temp = sub5BandDatesVector;
				    }
				    int i = Integer.parseInt(myHM.get(tempRA[ii]).toString());
				    if(Double.parseDouble(jt.getValueAt(i,1).toString())!=prevDate){
					int eventid = i+1;
					writeCommonData(dos, i, 1, (Group)tempO, eventid);
					if(sameDate.containsValue(tempRA[ii])){
                                            printPrev(dos,tempRA[ii],(Group)tempO);
					    prevDate = Double.parseDouble(jt.getValueAt(i,1).toString());
					}
				    }
				}
			    }
		    }

		} //end if check to see if dates vector has something in it


		if(k == 0){
		    dataStreamPrintString(dos, "\n\n</main>\n\n");
		}
		else if(k == 1){
		    dataStreamPrintString(dos, "\n\n</sub1>\n\n");
		}
		else if(k == 2){
		    dataStreamPrintString(dos, "\n\n</sub2>\n\n");
		}
		else if(k == 3){
		    dataStreamPrintString(dos, "\n\n</sub3>\n\n");
		}
		else if(k == 4){
		    dataStreamPrintString(dos, "\n\n</sub4>\n\n");
		}
		else if(k == 5){
		    dataStreamPrintString(dos, "\n\n</sub5>\n\n");
		}
	    }// end for looping through bands

	    dataStreamPrintString(dos, "\n\n<range>\n\n");
	    Vector t = new Vector();
	    for(int i = 0;i<=rows;i++){

//		Add second check condition so that hidden ranges do not get added twice
//		if(jt.getValueAt(i,4).toString().equals("range")){
		if(jt.getValueAt(i,4).toString().equals("range") && jt.getValueAt(i,15).toString().equalsIgnoreCase("no")){

		    t.add(new TLEvent(i,Integer.parseInt(jt.getValueAt(i,1).toString())));
		}
	    }

	    Object[] n = t.toArray();
	    Arrays.sort(n,
			new Comparator(){
				public int compare(Object a, Object b) {
				    TLEvent n = (TLEvent) a;
				    TLEvent o = (TLEvent) b;
				    int lastCmp = o.date().compareTo(n.date());
				    return (lastCmp!=0 ? lastCmp : o.rowID().compareTo(n.rowID()));
				}
			    }
			);

	    for(int k = n.length - 1;k>=0;k--){
		int i = Integer.parseInt(((TLEvent) n[k]).rowID());
		//System.out.println("here");
		int eventid = i+1;
		writeCommonData(dos, i, 2, null, eventid);
	    }

	    dataStreamPrintString(dos, "\n\n</range>\n\n");
	    dataStreamPrintString(dos, "\n\n<hiddenEvents>\n\n");
	    for(int i = 0;i<=rows;i++){
		if(jt.getValueAt(i,15).toString().equalsIgnoreCase("yes")){
		    int eventid = i+1;
		    writeCommonData(dos, i, 0, null, eventid);
		}
	    }
	    dataStreamPrintString(dos, "\n\n</hiddenEvents>\n\n");
	    dataStreamPrintString(dos, "\t\t<eventCount>" + rows + "</eventCount>\n");
	    dataStreamPrintString(dos, "\t</timelinedata>\n");
	    dataStreamPrintString(dos, "\n</timeline>");

	    //bw.flush();
	    //bw.close();

	    dos.flush();
	    dos.close();
	    bos.close();
	    fos.close();
	} catch (IOException ioe) {};
	//this last section closes file
	IsTableEdited.tableChanged = false;
	//creator.currentFileName = justname;
    }

    /**
     * There are 4 places in the code where timeline data is written, and only a few lines differ
     * so this writes the common parts to avoid duplication.
     *
     * @param dos DataOutputStream to write on
     * @param ii table row
     * @param s switch to choose which section differs
     * @param g Group if middate and group id info are needed
     * @param eid eventID writing to
     */
    public void writeCommonData(DataOutputStream dos, int ii, int s, Group g, int eid) {
	try {
	    dataStreamPrintString(dos, "\t\t<event id = \"" + eid + "\">\n");
	    dataStreamPrintString(dos, "\t\t\t<hide>" + jt.getValueAt(ii,15).toString().toLowerCase() + "</hide>\n");
	    dataStreamPrintString(dos, "\t\t\t<title1>" + modAmpsOnObject(jt.getValueAt(ii,0)) + "</title1>\n");
	    dataStreamPrintString(dos, "\t\t\t<title2></title2>\n");
	    dataStreamPrintString(dos, "\t\t\t<title3></title3>\n");
	    if(jt.getValueAt(ii,16) != "")
	    	dataStreamPrintString(dos, "\t\t\t<imagefile>" + modAmpsOnObject(jt.getValueAt(ii,9 + Integer.parseInt(jt.getValueAt(ii,16).toString() ) )) + "</imagefile>\n");
	    else
	    	dataStreamPrintString(dos, "\t\t\t<imagefile></imagefile>\n");
	    dataStreamPrintString(dos, "\t\t\t<iconid>" + jt.getValueAt(ii,4) + "</iconid>\n");
	    switch(s) {
	    case 0:
		dataStreamPrintString(dos, "\t\t\t<groupnum middate=\"" + jt.getValueAt(ii,1) + "\">0</groupnum>\n");
		dataStreamPrintString(dos, "\t\t\t<startdata>" + jt.getValueAt(ii,1) + "</startdata>\n");
		dataStreamPrintString(dos, "\t\t\t<enddata>" + jt.getValueAt(ii,2) + "</enddata>\n");
		dataStreamPrintString(dos, "\t\t\t<band>" + jt.getValueAt(ii,3) + "</band>\n");
		dataStreamPrintString(dos, "\t\t\t<hidedetail>" + jt.getValueAt(ii,5).toString().toLowerCase() + "</hidedetail>\n");
		break;
	    case 1:
		dataStreamPrintString(dos, "\t\t\t<groupnum middate=\"" + g.mid().getDate() + "\">" + g.getID() + "</groupnum>\n");
		dataStreamPrintString(dos, "\t\t\t<startdata>" + jt.getValueAt(ii,1) + "</startdata>\n");
		dataStreamPrintString(dos, "\t\t\t<enddata>" + jt.getValueAt(ii,2) + "</enddata>\n");
		dataStreamPrintString(dos, "\t\t\t<band>" + jt.getValueAt(ii,3) + "</band>\n");
		dataStreamPrintString(dos, "\t\t\t<hidedetail>" + jt.getValueAt(ii,5).toString().toLowerCase() + "</hidedetail>\n");
		break;
	    case 2:
		dataStreamPrintString(dos, "\t\t\t<groupnum middate=\"" + jt.getValueAt(ii,1) + "\">0</groupnum>\n");
		dataStreamPrintString(dos, "\t\t\t<startdata>" + jt.getValueAt(ii,1) + "</startdata>\n");
		dataStreamPrintString(dos, "\t\t\t<enddata>" + jt.getValueAt(ii,2) + "</enddata>\n");
		dataStreamPrintString(dos, "\t\t\t<band>main</band>\n");
		dataStreamPrintString(dos, "\t\t\t<hidedetail>" + jt.getValueAt(ii,5).toString().toLowerCase() + "</hidedetail>\n");
		break;
	    default:
		dataStreamPrintString(dos, "\t\t\t<groupnum middate=\"" + jt.getValueAt(ii,1) + "\">0</groupnum>\n");
		dataStreamPrintString(dos, "\t\t\t<startdata>" + jt.getValueAt(ii,1) + "</startdata>\n");
		dataStreamPrintString(dos, "\t\t\t<enddata>" + jt.getValueAt(ii,2) + "</enddata>\n");
		dataStreamPrintString(dos, "\t\t\t<band>" + jt.getValueAt(ii,3) + "</band>\n");
		dataStreamPrintString(dos, "\t\t\t<hidedetail>" + jt.getValueAt(ii,5).toString().toLowerCase() + "</hidedetail>\n");
		break;
	    }
	    dataStreamPrintString(dos, "\t\t\t<detail>\n");
	    dataStreamPrintString(dos, "\t\t\t\t<detailimagefile1>" + modAmpsOnObject(jt.getValueAt(ii,10)) + "</detailimagefile1>\n");
	    dataStreamPrintString(dos, "\t\t\t\t<detailimagefile2>" + modAmpsOnObject(jt.getValueAt(ii,11)) + "</detailimagefile2>\n");
	    dataStreamPrintString(dos, "\t\t\t\t<detailimagefile3>" + modAmpsOnObject(jt.getValueAt(ii,12)) + "</detailimagefile3>\n");
	    dataStreamPrintString(dos, "\t\t\t\t<detailimagefile4>" + modAmpsOnObject(jt.getValueAt(ii,13)) + "</detailimagefile4>\n");
	    dataStreamPrintString(dos, "\t\t\t\t<detailtext1>" + modAmpsOnObject(jt.getValueAt(ii,6)) + "</detailtext1>\n");
	    dataStreamPrintString(dos, "\t\t\t\t<detailtext2>" + modAmpsOnObject(jt.getValueAt(ii,7)) + "</detailtext2>\n");
	    dataStreamPrintString(dos, "\t\t\t\t<detailtext3>" + modAmpsOnObject(jt.getValueAt(ii,8)) + "</detailtext3>\n");
	    dataStreamPrintString(dos, "\t\t\t\t<detailtext4>" + modAmpsOnObject(jt.getValueAt(ii,9)) + "</detailtext4>\n");
	    // \n ensures that XML will write entire description, instead of cutting off at 1005 chars
	    // EDIT (02/02/2005) This has been commented out, and the 1005 character issue has not arisen,
	    // but the code will be left in just in case.

	    // if there is no description, then we cannot trim it
	    String jtt = new String();
	    if(jt.getValueAt(ii, 14) != null) {

		jtt = modAmpsOnObject(jt.getValueAt(ii, 14));
		jtt.trim();
		//jtt += "\n";
	    }

	    dataStreamPrintString(dos, "\t\t\t\t<description>" + jtt + "</description>\n");
	    dataStreamPrintString(dos, "\t\t\t</detail>\n");
	    dataStreamPrintString(dos, "\t\t</event>\n");
	} catch (IOException ioe) {
	    // oops
	}

}

    /**
     * Writes datapoint to DataOutput Stream
     *
     * @param dos DataOutputStream to write to
     * @param date date to write
     * @param g the group
     * @return returns success as a boolean value
     */
    public boolean printPrev(DataOutputStream dos,Double date,Group g){
	for(int ii = 0;ii<sameDatesArray.length;ii++){
	    if(new Double(sameDate.get(new Integer(sameDatesArray[ii].toString())).toString()).equals(date)){
		int i = Integer.parseInt(sameDatesArray[ii].toString());
		int eventid = i+1;
		writeCommonData(dos, i, 1, g, eventid);
	    }
	}
	return true;
    }

    /**
     * Output input string onto DataOutputStream 'dos'.
     *
     * @param dos DataOutputStream to output onto
     * @param str String to output
     */
    public void dataStreamPrintString(DataOutputStream dos, String str) throws IOException{
	dos.write(str.getBytes("UTF-8"));
    }

    /**
     * Returns the RGB value as a string of button selected.
     *
     * @param colorButton JButton of color being selected
     */
    public String returnRGB(JButton colorButton){
    	final JButton myColorButton = colorButton;

    	String a,b,c;

        a = Integer.toHexString(myColorButton.getBackground().getRed()).toUpperCase();
        b = Integer.toHexString(myColorButton.getBackground().getGreen()).toUpperCase();
        c = Integer.toHexString(myColorButton.getBackground().getBlue()).toUpperCase();

    	if(a.length() < 2){
	    a = "0" + a;
    	}
    	if(b.length() < 2){
	    b = "0" + b;
    	}
    	if(c.length() < 2){
	    c = "0" + c;
    	}
    	return a + b + c;
    }
}
