package edu.jhu.cer.TimelineCreator;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFileChooser;
import java.io.IOException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.util.HashMap;
import java.util.Vector;
import java.util.ArrayList;
import java.lang.NumberFormatException;

/**
 * Uses JFileChooser pkg to save file...String filename is used to write file
 */
class SaveMenuHandler implements ActionListener {
    /**
     * A reference to the main {@link TimelineCreator} object, which gets
     * used when we want to bring up a warning window.
     */
    TimelineCreator tlc;

    /**
     * Stores a reference to the timeline creator. 
     * @see #tlc tlc
     */
    public SaveMenuHandler(TimelineCreator creator) {
	tlc = creator;
    }

    /**
     * Public method for class.
     *
     * @param ae ActionEvent
     */
    public void actionPerformed(ActionEvent ae) {
	//    System.out.println(ae.getActionCommand()+" "+ae.getModifiers()+" " +ae.paramString());
	// check that all the data is logically correct
	doFoolProof();

	if(tlc.proceed){
	    JFileChooser jfc = new JFileChooser(System.getProperty("user.dir"));
	    File myfile = new File("timeline.xml");
	    jfc.setSelectedFile(myfile);
	    
	    int choice = jfc.showSaveDialog(tlc);
	    if (choice == JFileChooser.APPROVE_OPTION) {
		
		if(jfc.getSelectedFile().exists()){
		    WarnOverwrite wo = new WarnOverwrite(jfc.getSelectedFile());
		}
		
		if(tlc.proceedOverwrite){
		    String filename = jfc.getSelectedFile().getAbsolutePath();
		    String justname = jfc.getSelectedFile().getName();               

		    // actually write the file here
		    TimelineDataWriter tldw = new TimelineDataWriter(tlc);
		    tldw.writeData(filename);
		}
	    }
	}
    }

    
    /**
     * Checks first that all necessary data for a timeline is present and in the correct form.
     * (start and end dates, units and gradation). 
     * Then, doFoolProof checks that ataStreamPrintString(dos, "\t\t\t<detail>\n");
     dataStreamPrintString(dos, "\t\t\t\t<detailimagefile>" + jt.getValueAt(i,10) + "</detailimagefile>\n");
     dataStreamPrintString(dos, "\t\t\t\t<detailtext1>" + jt.getValueAt(i,6) + "</detailtext1>\n");
     dataStreamPrintString(dos, "\t\t\t\t<detailtext2>" + jt.getValueAt(i,7) + "</detailtext2>\n");
     dataStreamPrintString(dos, "\t\t\t\t<detailtext3>" + jt.getValueAt(i,8) + "</detailtext3>\n");
     dataStreamPrintString(dos, "\t\t\t\t<detailtext4>" + jt.getValueAt(i,9) + "</detailtext4>\n");
     // \n ensures that XML will write entire description, instead of cutting off at 1005 chars
     String jtt = new String();
     jtt = ((String)jt.getValueAt(i, 11)).trim();
     jtt += "\n";
     dataStreamPrintString(dos, "\t\t\t\t<description>" + jtt + "</description>\n");
     dataStreamPrintString(dos, "\t\t\t</detail>\n");
     dataStreamPrintString(dos, "\t\t</event>\n"); all the data is logically correct. That is, start dates come before
     * end dates, events on the timeline don't have start or end dates outside the bounds
     * of the timeline itself. If errors are found a window is shown describing these errors.
     */
    public void doFoolProof(){
	int warnCount = 0;
	ArrayList messages = new ArrayList();
	int rows = tlc.jt.getRowCount() - 1;
	int actualRow;
	boolean checkStartEnd = true;
	boolean checkTLStartEnd = true;

	// store the start and end dates of the timeline as a whole
	double tl_start = 0;
	double tl_end = 0;

	// store the start and end dates for events
	double event_start = 0;
	double event_end = 0;

	// sets the Timeline.proceed value to false just for good measure
	tlc.proceed = false;

	/*****
	 *** First check that all basic Timeline features are present.
	 *****/

	//timeline start is present and its a proper double type
	try {
	    Double TLstartD = new Double(tlc.configStartTF.getText());
	    tl_start = TLstartD.doubleValue();
	}
	catch (NumberFormatException nfe4) {
	    messages.add("Please set the timeline start date on the interface configuration tab");
	    warnCount++;
	    checkTLStartEnd = false;
	}
		
	//timeline end is present and its a proper double type
	try {
	    Double TLendD = new Double(tlc.configStopTF.getText());
	    tl_end = TLendD.doubleValue();
	}
	catch (NumberFormatException nfe5){
	    messages.add("Please set the timeline end date on the interface configuration tab.");
	    warnCount++;
	    checkTLStartEnd = false;
	}
		
	//timeline start is less than or equal to timeline end
	if(checkTLStartEnd){
	    if(tl_end < tl_start){
		messages.add("The timeline start date is greater than the end date in the settings.");
		warnCount++;
	    }
	}
		
	//timeline gradation is present and its a proper double type
	try{
	    Double TLSpacing = new Double(tlc.configSpacingTF.getText());
	}
	catch (NumberFormatException nfe6){
	    messages.add("Please set the timeline gradation on the interface configuration tab.");
	    warnCount++;
	    checkTLStartEnd = false;
	}
		
	//timeline units is present
	if(tlc.configUnitsTF.getText().equals("")) {
	    messages.add("Please set the timeline units on the interface configuration tab.");
	    warnCount++;
	}	
	     
	/*****
	 *** Now check that each row is valid.
	 *****/
		
	for(int i = 0; i <= rows; i++){
	    checkStartEnd = true;
	    actualRow = i+1;
	    //start date is present and its a proper double type
	    try {
		Double startD = new Double(tlc.jt.getValueAt(i,1).toString());
		event_start = startD.doubleValue();
	    }
	    catch (NumberFormatException nfe1){
		String eventName = tlc.jt.getValueAt(i,0).toString();
		messages.add("For event \""+eventName+"\", the start date is not valid.");
		warnCount++;
		checkStartEnd = false;
	    }
		
	    // make sure the start date of this event is within the general timeline range
	    if(tl_start > event_start) {
		String eventName = tlc.jt.getValueAt(i,0).toString();
		messages.add("For event \""+eventName+"\", the event start date is before the timeline start date.");
		warnCount++;
	    }
		
	    if(tl_end < event_start) {
		String eventName = tlc.jt.getValueAt(i,0).toString();
		messages.add("For event \""+eventName+"\", the event start date is after the timeline end date.");
		warnCount++;
	    }

	    // check that end date is present and in proper double type
	    try{
		if(tlc.jt.getValueAt(i,2)!=null){
		    Double endD = new Double(tlc.jt.getValueAt(i,2).toString());
		    event_end = endD.doubleValue();
		} else {
		    checkStartEnd = false;
		}
	    }
	    catch (NumberFormatException nfe2){
		//if end date present, its a proper double type
		if(!tlc.jt.getValueAt(i,2).toString().equals("")){
			String eventName = tlc.jt.getValueAt(i,0).toString();
			messages.add("For event \""+eventName+"\", the end date is not valid.");
			warnCount++;
			checkStartEnd = false;
		} else {
		    // this is just an empty cell in the table, so ignore it
		    checkStartEnd = false;
		}
		//if event type is range, both start date & end date are present 
		if(tlc.jt.getValueAt(i,4).toString().equals("range")){
			String eventName = tlc.jt.getValueAt(i,0).toString();
			messages.add("For event \""+eventName+"\", event described as a range must have both start and an end date.");
			warnCount++;
		}
	    }				

	    //if end date present, end date > start date
	    if(checkStartEnd){
		if(event_end < event_start){
		    String eventName = tlc.jt.getValueAt(i,0).toString();
		    messages.add("For event \""+eventName+"\", the event end date comes before its own start date.");
		    warnCount++;
		}

		// make sure the end date of this event is within the general timeline range
		if(tl_start > event_end) {
		    String eventName = tlc.jt.getValueAt(i,0).toString();
		    messages.add("For event \""+eventName+"\", the event end date is before the timeline start date.");
		    warnCount++;
		}
		    
		if(tl_end < event_end) {
		    String eventName = tlc.jt.getValueAt(i,0).toString();
		    messages.add("For event \""+eventName+"\", the event end date is after the timeline end date.");
		    warnCount++;
		}
	    }

	    //if event added to a band, make sure its given a name
	    if(tlc.jt.getValueAt(i,3).toString().equals("sub1") && tlc.configBand1TitleTF.getText().equals("")){
		String eventName = tlc.jt.getValueAt(i,0).toString();
		messages.add("For event \""+eventName+"\", event is added to Sub Band 1, but Sub Band 1 is undefined in the Settings.");
		warnCount++;
	    }
	    else if(tlc.jt.getValueAt(i,3).toString().equals("sub2") && tlc.configBand2TitleTF.getText().equals("")){
		String eventName = tlc.jt.getValueAt(i,0).toString();
		messages.add("For event \""+eventName+"\", event is added to Sub Band 2, but Sub Band 2 is undefined in the Settings.");
		warnCount++;
	    }
	    else if(tlc.jt.getValueAt(i,3).toString().equals("sub3") && tlc.configBand3TitleTF.getText().equals("")){
		String eventName = tlc.jt.getValueAt(i,0).toString();
		messages.add("For event \""+eventName+"\", event is added to Sub Band 3, but Sub Band 3 is undefined in the Settings.");
		warnCount++;
	    }
	    else if(tlc.jt.getValueAt(i,3).toString().equals("sub4") && tlc.configBand4TitleTF.getText().equals("")){
		String eventName = tlc.jt.getValueAt(i,0).toString();
		messages.add("For event \""+eventName+"\", event is added to Sub Band 4, but Sub Band 4 is undefined in the Settings.");
		warnCount++;
	    }	
	    else if(tlc.jt.getValueAt(i,3).toString().equals("sub5") && tlc.configBand5TitleTF.getText().equals("")){
		String eventName = tlc.jt.getValueAt(i,0).toString();
		messages.add("For event \""+eventName+"\", event is added to Sub Band 5, but Sub Band 5 is undefined in the Settings.");
		warnCount++;
	    }					
	}

	// if there are warnings, then bring up a window saying so

	if(warnCount > 0){
	    WarnDialog wd = new WarnDialog(messages);
	} else {
	    tlc.proceed = true;
	}
    }
}
